@@ -276,16 +276,18 @@ static void ipa_cldap_process(struct ipa_cldap_ctx *ctx,
     LOG_TRACE("CLDAP Request received");
 
     ret = ipa_cldap_netlogon(ctx, req, &reply);
+
+done:
     if (ret != 0) {
         /* bad request, or internal error, return empty reply */
         /* as Windows does per MS-ADTS 6.3.3.3 */
         memset(&reply, 0, sizeof(struct berval));
     }
 
-done:
     ipa_cldap_respond(ctx, req, &reply);
 
     ipa_cldap_free_kvps(&req->kvps);
+    free(reply.bv_val);
     free(req);
     return;
 }
@@ -314,11 +316,12 @@ static struct ipa_cldap_req *ipa_cldap_recv_dgram(struct ipa_cldap_ctx *ctx)
     return req;
 }
 
-void *ipa_cldap_worker(struct ipa_cldap_ctx *ctx)
+void *ipa_cldap_worker(void *arg)
 {
     struct ipa_cldap_req *req;
     struct pollfd fds[2];
     bool stop = false;
+    struct ipa_cldap_ctx *ctx = (struct ipa_cldap_ctx *) arg;
     int ret;
 
     while (!stop) {
