@@ -155,13 +155,13 @@ IPA.opened_dialogs = {
         this.app = app;
 
         this.handlers.push(topic.subscribe('dialog.open',
-            lang.hitch(this, this.on_dialog_open)));
+            this.on_dialog_open.bind(this)));
 
         this.handlers.push(topic.subscribe('dialog.opened',
-            lang.hitch(this, this.on_dialog_opened)));
+            this.on_dialog_opened.bind(this)));
 
         this.handlers.push(topic.subscribe('dialog.closed',
-            lang.hitch(this, this.on_dialog_closed)));
+            this.on_dialog_closed.bind(this)));
     }
 };
 
@@ -185,6 +185,8 @@ IPA.dialog_button = function(spec) {
     that.visible = spec.visible !== undefined ? spec.visible : true;
     /** @property {boolean} enabled=true Button is enabled */
     that.enabled = spec.enabled !== undefined ? spec.enabled : true;
+    /** @property {String} button's css classes */
+    that.button_class = spec.button_class || 'btn btn-default';
     /** @property {jQuery} element Button element */
     that.element = null;
 
@@ -407,6 +409,7 @@ IPA.dialog = function(spec) {
 
         that.title_close_button = $('<button/>', {
             'class': 'close',
+            'type': 'button',
             'aria-hidden': 'true',
             click: function() {
                 that.close();
@@ -494,6 +497,7 @@ IPA.dialog = function(spec) {
             var ui_button = IPA.button({
                 name: button.name,
                 label: button.label,
+                button_class: button.button_class,
                 disabled: !button.enabled,
                 click: button.click
             });
@@ -572,11 +576,11 @@ IPA.dialog = function(spec) {
         this.emit('open', { source: that });
         topic.publish('dialog.open', { source: that });
 
-        this.show(lang.hitch(this, function() {
+        this.show(function() {
             that.focus_first_element();
             that.emit('opened', { source: that });
             topic.publish('dialog.opened', { source: that });
-        }));
+        }.bind(this));
     };
 
     /**
@@ -672,12 +676,12 @@ IPA.dialog = function(spec) {
         if (!that.dom_node) return;
 
         var dom_node = that.dom_node;
-        this.hide(lang.hitch(this, function() {
+        this.hide(function() {
             dom_node.remove();
             that.dom_node = null;
             that.emit('closed', { source: that });
             topic.publish('dialog.closed', { source: that });
-        }));
+        }.bind(this));
     };
 
     /**
@@ -797,6 +801,7 @@ IPA.adder_dialog = function(spec) {
 
     spec = spec || {};
 
+    spec.title = spec.title || '@i18n:buttons.add';
     spec.name = spec.name || 'adder_dialog';
 
     var that = IPA.dialog(spec);
@@ -914,35 +919,7 @@ IPA.adder_dialog = function(spec) {
             'class': 'input-group col-md-12 adder-dialog-top'
         }).appendTo(container);
 
-        var filter_placeholder = text.get('@i18n:association.filter_placeholder');
-        filter_placeholder = filter_placeholder.replace('${other_entity}',
-            that.other_entity.metadata.label);
-
-        that.filter_field = $('<input/>', {
-            type: 'text',
-            name: 'filter',
-            'class': 'form-control',
-            'placeholder': filter_placeholder,
-            keyup: function(event) {
-                if (event.keyCode === keys.ENTER) {
-                    that.search();
-                    return false;
-                }
-            }
-        }).appendTo(input_group);
-
-        var input_group_btn = $('<div/>', {
-            'class': 'input-group-btn'
-        }).appendTo(input_group);
-
-        that.find_button = IPA.button({
-            name: 'find',
-            label: '@i18n:buttons.filter',
-            click: function() {
-                that.search();
-                return false;
-            }
-        }).appendTo(input_group_btn);
+        that.filter_field = that.get_filter_field(input_group);
 
         var row = $('<div/>', { 'class': 'row adder-dialog-main'}).appendTo(container);
         //
@@ -1127,6 +1104,49 @@ IPA.adder_dialog = function(spec) {
         return that.filter_field.val();
     };
 
+    /**
+     * Return field for filtering available items
+     *
+     * Default implementation returns text input + "Filter" button.
+     * It can be overridden.
+     *
+     * @param {HTMLElement} input_group - container for a filter field
+     * @return {HTMLElement}
+     */
+    that.get_filter_field = function(input_group) {
+        var filter_placeholder = text.get(
+            '@i18n:association.filter_placeholder'
+        ).replace('${other_entity}', that.other_entity.metadata.label);
+
+        var filter_field = $('<input/>', {
+            type: 'text',
+            name: 'filter',
+            'class': 'form-control',
+            'placeholder': filter_placeholder,
+            keyup: function(event) {
+                if (event.keyCode === keys.ENTER) {
+                    that.search();
+                    return false;
+                }
+            }
+        }).appendTo(input_group);
+
+        var input_group_btn = $('<div/>', {
+            'class': 'input-group-btn'
+        }).appendTo(input_group);
+
+        that.find_button = IPA.button({
+            name: 'find',
+            label: '@i18n:buttons.filter',
+            click: function() {
+                that.search();
+                return false;
+            }
+        }).appendTo(input_group_btn);
+
+        return filter_field;
+    };
+
     /**
      * Clear rows in available table
      */
@@ -1305,7 +1325,7 @@ IPA.about_dialog = function(spec) {
     spec = spec || {};
 
     spec.name = spec.name || 'version_dialog';
-    var product = 'FreeIPA';
+    var product = 'IPA';
     var version = 'Unknown';
     var msg = text.get('@i18n:dialogs.about_message', '${product}, version: ${version}');
     if (IPA.env) {
@@ -1359,6 +1379,12 @@ IPA.confirm_dialog = function(spec) {
     /** @property {Function} cancel_label Cancel button label */
     that.cancel_label = text.get(spec.cancel_label || '@i18n:buttons.cancel');
 
+    /** @property {String} on_ok css class */
+    that.ok_button_class = spec.ok_button_class || 'btn btn-default';
+
+    /** @property {String} on_cancel css class */
+    that.cancel_button_class = spec.cancel_button_class || 'btn btn-default';
+
     /**
      * Dialog is confirmed
      * @protected
@@ -1416,6 +1442,7 @@ IPA.confirm_dialog = function(spec) {
         that.create_button({
             name: 'ok',
             label: that.ok_label,
+            button_class: that.ok_button_class,
             click: function() {
                 that.on_confirm();
             }
@@ -1424,6 +1451,7 @@ IPA.confirm_dialog = function(spec) {
         that.create_button({
             name: 'cancel',
             label: that.cancel_label,
+            button_class: that.cancel_button_class,
             click: function() {
                 that.confirmed = false;
                 that.close();
@@ -1439,6 +1467,143 @@ IPA.confirm_dialog = function(spec) {
     return that;
 };
 
+/**
+ * Custom command multivalued dialog
+ *
+ * Combines confirmation dialog which could be reopen after unsucessful command
+ * call. It also allows to define fields and widgets in the dialog.
+ *
+ * Acceptation is done by clicking on 'Add' button or hitting 'ENTER' key,
+ * refusal by clicking on 'Cancel' button or hitting 'ESCAPE' key.
+ *
+ * @class
+ * @extends IPA.form_dialog
+ */
+IPA.custom_command_multivalued_dialog = function(spec) {
+    spec = spec || {};
+
+    /**
+     * Name of confirmation button, by default set to 'Add'.
+     * @param {String} ok_label
+     */
+    spec.ok_label = spec.ok_label || '@i18n:buttons.add';
+
+    var that = IPA.form_dialog(spec);
+
+    that.close = function() {
+        that.dialog_close();
+    };
+
+    that.on_confirm = function() {
+        if (that.on_ok) that.on_ok();
+    };
+
+    return that;
+};
+
+
+/**
+ *
+ *
+ *
+ *
+ *
+ *
+ */
+IPA.table_mixin = function() {
+
+    return {
+        mixin: {
+            /**
+             * Create title above a table.
+             *
+             * @param {string} cls css class which will be added to current title
+             */
+            create_title: function(str, cls) {
+                cls = cls || '';
+
+                return $('<h3 />', {
+                    'class': cls,
+                    text: text.get(str)
+                });
+            },
+
+            /**
+             * Create table layout..
+             *
+             * @param {string} cls css class which will be added to current table layout
+             */
+            create_layout: function(cls) {
+                cls = cls || '';
+
+                return $('<div />', {
+                    'class': 'table-layout ' + cls
+                });
+            },
+
+            /**
+             * Create one row to the table layout.
+             *
+             * @param {string} cls css class which will be added to current row
+             */
+            create_row: function(cls) {
+                cls = cls || '';
+
+                return $('<div />', {
+                    'class': 'table-row ' + cls
+                });
+            },
+
+            /**
+             * Create one cell to the table layout.
+             *
+             * @param {string} string, will be parsed using our provider
+             * @param {string} suffix, string which will be concatenated to the end of
+             *                  'str' string. Not parsed using text.get()
+             * @param {string} cls css class which will be added to current cell
+             * @param {string} nm css name which will be added to current cell
+             */
+            create_cell: function(str, suffix, cls, nm) {
+                str = str || '';
+                suffix = suffix || '';
+                cls = cls || '';
+                nm = nm || '';
+
+                return $('<div />', {
+                    'class': 'table-cell ' + cls,
+                    name: nm,
+                    text: text.get(str) + suffix
+                });
+            },
+
+            /**
+             * Create header cell to the table layout.
+             *
+             * @param {string} string, will be parsed using our provider
+             * @param {string} suffix, string which will be concatenated to the end of
+             *                  'str' string. Not parsed using text.get()
+             * @param {string} cls css class which will be added to current cell
+             */
+            create_header_cell: function(str, suffix, cls) {
+                str = str || '';
+                suffix = suffix || '';
+                cls = cls || '';
+
+                return $('<div />', {
+                    'class': 'table-cell table-head' + cls,
+                    text: text.get(str) + suffix
+                });
+            }
+        },
+
+        apply: function(obj) {
+            $.extend(obj, this.mixin);
+        }
+    };
+};
+
+
+
 /**
  * General form dialog with confirmation feature
  * @class  dialog.form_dialog
