@@ -63,6 +63,22 @@ return {
                 }
             ]
         },
+        {
+            $type: 'association',
+            name: 'membermanager_group',
+            add_method: 'add_member_manager',
+            remove_method: 'remove_member_manager',
+            add_title: '@i18n:objects.hostgroup.add_membermanager_group',
+            remove_title: '@i18n:objects.hostgroup.remove_membermanager_group'
+        },
+        {
+            $type: 'association',
+            name: 'membermanager_user',
+            add_method: 'add_member_manager',
+            remove_method: 'remove_member_manager',
+            add_title: '@i18n:objects.hostgroup.add_membermanager_user',
+            remove_title: '@i18n:objects.hostgroup.remove_membermanager_user'
+        },
         {
             $type: 'association',
             name: 'member_host',
