@@ -41,9 +41,30 @@ IPA provides a designated binddn to use with Sudo located at:
 uid=sudo,cn=sysaccounts,cn=etc,dc=example,dc=com
 
 To enable the binddn run the following command to set the password:
-LDAPTLS_CACERT=/etc/ipa/ca.crt /usr/bin/ldappasswd -S -W -h ipa.example.com -ZZ -D "cn=Directory Manager" uid=sudo,cn=sysaccounts,cn=etc,dc=example,dc=com
+LDAPTLS_CACERT=/etc/ipa/ca.crt /usr/bin/ldappasswd -S -W \\
+    -H ldap://ipa.example.com -ZZ -D "cn=Directory Manager" \\
+    uid=sudo,cn=sysaccounts,cn=etc,dc=example,dc=com
 
-For more information, see the IPA Documentation to Sudo.
+EXAMPLES:
+
+ Create a new rule:
+   ipa sudorule-add readfiles
+
+ Add sudo command object and add it as allowed command in the rule:
+   ipa sudocmd-add /usr/bin/less
+   ipa sudorule-add-allow-command readfiles --sudocmds /usr/bin/less
+
+ Add a host to the rule:
+   ipa sudorule-add-host readfiles --hosts server.example.com
+
+ Add a user to the rule:
+   ipa sudorule-add-user readfiles --users jsmith
+
+ Add a special Sudo rule for default Sudo server configuration:
+   ipa sudorule-add defaults
+
+ Set a default Sudo option:
+   ipa sudorule-add-option defaults --sudooption '!authenticate'
 """)
 
 register = Registry()
@@ -113,6 +134,12 @@ class sudorule(Object):
             required=False,
             label=_(u'User Groups'),
         ),
+        parameters.Str(
+            'externaluser',
+            required=False,
+            label=_(u'External User'),
+            doc=_(u'External User the rule applies to (sudorule-find only)'),
+        ),
         parameters.Str(
             'memberhost_host',
             required=False,
@@ -123,6 +150,17 @@ class sudorule(Object):
             required=False,
             label=_(u'Host Groups'),
         ),
+        parameters.Str(
+            'hostmask',
+            multivalue=True,
+            label=_(u'Host Masks'),
+        ),
+        parameters.Str(
+            'externalhost',
+            required=False,
+            multivalue=True,
+            label=_(u'External host'),
+        ),
         parameters.Str(
             'memberallowcmd_sudocmd',
             required=False,
@@ -155,18 +193,24 @@ class sudorule(Object):
             label=_(u'Groups of RunAs Users'),
             doc=_(u'Run as any user within a specified group'),
         ),
-        parameters.Str(
-            'externaluser',
-            required=False,
-            label=_(u'External User'),
-            doc=_(u'External User the rule applies to (sudorule-find only)'),
-        ),
         parameters.Str(
             'ipasudorunasextuser',
             required=False,
             label=_(u'RunAs External User'),
             doc=_(u'External User the commands can run as (sudorule-find only)'),
         ),
+        parameters.Str(
+            'ipasudorunasextusergroup',
+            required=False,
+            label=_(u'External Groups of RunAs Users'),
+            doc=_(u'External Groups of users that the command can run as'),
+        ),
+        parameters.Str(
+            'ipasudorunasgroup_group',
+            required=False,
+            label=_(u'RunAs Groups'),
+            doc=_(u'Run with the gid of a specified POSIX group'),
+        ),
         parameters.Str(
             'ipasudorunasextgroup',
             required=False,
@@ -178,18 +222,6 @@ class sudorule(Object):
             required=False,
             label=_(u'Sudo Option'),
         ),
-        parameters.Str(
-            'ipasudorunasgroup_group',
-            required=False,
-            label=_(u'RunAs Groups'),
-            doc=_(u'Run with the gid of a specified POSIX group'),
-        ),
-        parameters.Str(
-            'externalhost',
-            required=False,
-            multivalue=True,
-            label=_(u'External host'),
-        ),
     )
 
 
@@ -271,6 +303,13 @@ class sudorule_add(Method):
             label=_(u'External User'),
             doc=_(u'External User the rule applies to (sudorule-find only)'),
         ),
+        parameters.Str(
+            'externalhost',
+            required=False,
+            multivalue=True,
+            label=_(u'External host'),
+            exclude=('cli', 'webui'),
+        ),
         parameters.Str(
             'ipasudorunasextuser',
             required=False,
@@ -285,13 +324,6 @@ class sudorule_add(Method):
             label=_(u'RunAs External Group'),
             doc=_(u'External Group the commands can run as (sudorule-find only)'),
         ),
-        parameters.Str(
-            'externalhost',
-            required=False,
-            multivalue=True,
-            label=_(u'External host'),
-            exclude=('cli', 'webui'),
-        ),
         parameters.Str(
             'setattr',
             required=False,
@@ -320,6 +352,13 @@ class sudorule_add(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
     )
     has_output = (
         output.Output(
@@ -330,9 +369,8 @@ class sudorule_add(Method):
         output.Entry(
             'result',
         ),
-        output.Output(
+        output.PrimaryKey(
             'value',
-            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
@@ -364,13 +402,20 @@ class sudorule_add_allow_command(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'sudocmd',
             required=False,
             multivalue=True,
             cli_name='sudocmds',
             label=_(u'member sudo command'),
-            doc=_(u'comma-separated list of sudo commands to add'),
+            doc=_(u'sudo commands to add'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -379,7 +424,7 @@ class sudorule_add_allow_command(Method):
             multivalue=True,
             cli_name='sudocmdgroups',
             label=_(u'member sudo command group'),
-            doc=_(u'comma-separated list of sudo command groups to add'),
+            doc=_(u'sudo command groups to add'),
             alwaysask=True,
         ),
     )
@@ -426,13 +471,20 @@ class sudorule_add_deny_command(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'sudocmd',
             required=False,
             multivalue=True,
             cli_name='sudocmds',
             label=_(u'member sudo command'),
-            doc=_(u'comma-separated list of sudo commands to add'),
+            doc=_(u'sudo commands to add'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -441,7 +493,7 @@ class sudorule_add_deny_command(Method):
             multivalue=True,
             cli_name='sudocmdgroups',
             label=_(u'member sudo command group'),
-            doc=_(u'comma-separated list of sudo command groups to add'),
+            doc=_(u'sudo command groups to add'),
             alwaysask=True,
         ),
     )
@@ -488,13 +540,20 @@ class sudorule_add_host(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'host',
             required=False,
             multivalue=True,
             cli_name='hosts',
             label=_(u'member host'),
-            doc=_(u'comma-separated list of hosts to add'),
+            doc=_(u'hosts to add'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -503,9 +562,15 @@ class sudorule_add_host(Method):
             multivalue=True,
             cli_name='hostgroups',
             label=_(u'member host group'),
-            doc=_(u'comma-separated list of host groups to add'),
+            doc=_(u'host groups to add'),
             alwaysask=True,
         ),
+        parameters.Str(
+            'hostmask',
+            required=False,
+            multivalue=True,
+            label=_(u'host masks of allowed hosts'),
+        ),
     )
     has_output = (
         output.Entry(
@@ -541,11 +606,41 @@ class sudorule_add_option(Method):
             cli_name='sudooption',
             label=_(u'Sudo Option'),
         ),
+        parameters.Flag(
+            'all',
+            doc=_(u'Retrieve and print all attributes from the server. Affects command output.'),
+            exclude=('webui',),
+            default=False,
+            autofill=True,
+        ),
+        parameters.Flag(
+            'raw',
+            doc=_(u'Print entries as stored on the server. Only affects output format.'),
+            exclude=('webui',),
+            default=False,
+            autofill=True,
+        ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
     )
     has_output = (
         output.Output(
+            'summary',
+            (unicode, type(None)),
+            doc=_(u'User-friendly description of action performed'),
+        ),
+        output.Entry(
             'result',
         ),
+        output.PrimaryKey(
+            'value',
+            doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
+        ),
     )
 
 
@@ -575,13 +670,20 @@ class sudorule_add_runasgroup(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'group',
             required=False,
             multivalue=True,
             cli_name='groups',
             label=_(u'member group'),
-            doc=_(u'comma-separated list of groups to add'),
+            doc=_(u'groups to add'),
             alwaysask=True,
         ),
     )
@@ -628,13 +730,20 @@ class sudorule_add_runasuser(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'user',
             required=False,
             multivalue=True,
             cli_name='users',
             label=_(u'member user'),
-            doc=_(u'comma-separated list of users to add'),
+            doc=_(u'users to add'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -643,7 +752,7 @@ class sudorule_add_runasuser(Method):
             multivalue=True,
             cli_name='groups',
             label=_(u'member group'),
-            doc=_(u'comma-separated list of groups to add'),
+            doc=_(u'groups to add'),
             alwaysask=True,
         ),
     )
@@ -690,13 +799,20 @@ class sudorule_add_user(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'user',
             required=False,
             multivalue=True,
             cli_name='users',
             label=_(u'member user'),
-            doc=_(u'comma-separated list of users to add'),
+            doc=_(u'users to add'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -705,7 +821,7 @@ class sudorule_add_user(Method):
             multivalue=True,
             cli_name='groups',
             label=_(u'member group'),
-            doc=_(u'comma-separated list of groups to add'),
+            doc=_(u'groups to add'),
             alwaysask=True,
         ),
     )
@@ -757,10 +873,8 @@ class sudorule_del(Method):
             dict,
             doc=_(u'List of deletions that failed'),
         ),
-        output.Output(
+        output.ListOfPrimaryKeys(
             'value',
-            unicode,
-            doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
 
@@ -776,6 +890,8 @@ class sudorule_disable(Method):
             label=_(u'Rule name'),
         ),
     )
+    takes_options = (
+    )
     has_output = (
         output.Output(
             'result',
@@ -794,6 +910,8 @@ class sudorule_enable(Method):
             label=_(u'Rule name'),
         ),
     )
+    takes_options = (
+    )
     has_output = (
         output.Output(
             'result',
@@ -885,6 +1003,13 @@ class sudorule_find(Method):
             label=_(u'External User'),
             doc=_(u'External User the rule applies to (sudorule-find only)'),
         ),
+        parameters.Str(
+            'externalhost',
+            required=False,
+            multivalue=True,
+            label=_(u'External host'),
+            exclude=('cli', 'webui'),
+        ),
         parameters.Str(
             'ipasudorunasextuser',
             required=False,
@@ -899,13 +1024,6 @@ class sudorule_find(Method):
             label=_(u'RunAs External Group'),
             doc=_(u'External Group the commands can run as (sudorule-find only)'),
         ),
-        parameters.Str(
-            'externalhost',
-            required=False,
-            multivalue=True,
-            label=_(u'External host'),
-            exclude=('cli', 'webui'),
-        ),
         parameters.Int(
             'timelimit',
             required=False,
@@ -932,6 +1050,13 @@ class sudorule_find(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Flag(
             'pkey_only',
             required=False,
@@ -1041,6 +1166,13 @@ class sudorule_mod(Method):
             label=_(u'External User'),
             doc=_(u'External User the rule applies to (sudorule-find only)'),
         ),
+        parameters.Str(
+            'externalhost',
+            required=False,
+            multivalue=True,
+            label=_(u'External host'),
+            exclude=('cli', 'webui'),
+        ),
         parameters.Str(
             'ipasudorunasextuser',
             required=False,
@@ -1055,13 +1187,6 @@ class sudorule_mod(Method):
             label=_(u'RunAs External Group'),
             doc=_(u'External Group the commands can run as (sudorule-find only)'),
         ),
-        parameters.Str(
-            'externalhost',
-            required=False,
-            multivalue=True,
-            label=_(u'External host'),
-            exclude=('cli', 'webui'),
-        ),
         parameters.Str(
             'setattr',
             required=False,
@@ -1104,6 +1229,13 @@ class sudorule_mod(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
     )
     has_output = (
         output.Output(
@@ -1114,9 +1246,8 @@ class sudorule_mod(Method):
         output.Entry(
             'result',
         ),
-        output.Output(
+        output.PrimaryKey(
             'value',
-            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
@@ -1148,13 +1279,20 @@ class sudorule_remove_allow_command(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'sudocmd',
             required=False,
             multivalue=True,
             cli_name='sudocmds',
             label=_(u'member sudo command'),
-            doc=_(u'comma-separated list of sudo commands to remove'),
+            doc=_(u'sudo commands to remove'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -1163,7 +1301,7 @@ class sudorule_remove_allow_command(Method):
             multivalue=True,
             cli_name='sudocmdgroups',
             label=_(u'member sudo command group'),
-            doc=_(u'comma-separated list of sudo command groups to remove'),
+            doc=_(u'sudo command groups to remove'),
             alwaysask=True,
         ),
     )
@@ -1210,13 +1348,20 @@ class sudorule_remove_deny_command(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'sudocmd',
             required=False,
             multivalue=True,
             cli_name='sudocmds',
             label=_(u'member sudo command'),
-            doc=_(u'comma-separated list of sudo commands to remove'),
+            doc=_(u'sudo commands to remove'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -1225,7 +1370,7 @@ class sudorule_remove_deny_command(Method):
             multivalue=True,
             cli_name='sudocmdgroups',
             label=_(u'member sudo command group'),
-            doc=_(u'comma-separated list of sudo command groups to remove'),
+            doc=_(u'sudo command groups to remove'),
             alwaysask=True,
         ),
     )
@@ -1272,13 +1417,20 @@ class sudorule_remove_host(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'host',
             required=False,
             multivalue=True,
             cli_name='hosts',
             label=_(u'member host'),
-            doc=_(u'comma-separated list of hosts to remove'),
+            doc=_(u'hosts to remove'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -1287,9 +1439,15 @@ class sudorule_remove_host(Method):
             multivalue=True,
             cli_name='hostgroups',
             label=_(u'member host group'),
-            doc=_(u'comma-separated list of host groups to remove'),
+            doc=_(u'host groups to remove'),
             alwaysask=True,
         ),
+        parameters.Str(
+            'hostmask',
+            required=False,
+            multivalue=True,
+            label=_(u'host masks of allowed hosts'),
+        ),
     )
     has_output = (
         output.Entry(
@@ -1325,11 +1483,41 @@ class sudorule_remove_option(Method):
             cli_name='sudooption',
             label=_(u'Sudo Option'),
         ),
+        parameters.Flag(
+            'all',
+            doc=_(u'Retrieve and print all attributes from the server. Affects command output.'),
+            exclude=('webui',),
+            default=False,
+            autofill=True,
+        ),
+        parameters.Flag(
+            'raw',
+            doc=_(u'Print entries as stored on the server. Only affects output format.'),
+            exclude=('webui',),
+            default=False,
+            autofill=True,
+        ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
     )
     has_output = (
         output.Output(
+            'summary',
+            (unicode, type(None)),
+            doc=_(u'User-friendly description of action performed'),
+        ),
+        output.Entry(
             'result',
         ),
+        output.PrimaryKey(
+            'value',
+            doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
+        ),
     )
 
 
@@ -1359,13 +1547,20 @@ class sudorule_remove_runasgroup(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'group',
             required=False,
             multivalue=True,
             cli_name='groups',
             label=_(u'member group'),
-            doc=_(u'comma-separated list of groups to remove'),
+            doc=_(u'groups to remove'),
             alwaysask=True,
         ),
     )
@@ -1412,13 +1607,20 @@ class sudorule_remove_runasuser(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'user',
             required=False,
             multivalue=True,
             cli_name='users',
             label=_(u'member user'),
-            doc=_(u'comma-separated list of users to remove'),
+            doc=_(u'users to remove'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -1427,7 +1629,7 @@ class sudorule_remove_runasuser(Method):
             multivalue=True,
             cli_name='groups',
             label=_(u'member group'),
-            doc=_(u'comma-separated list of groups to remove'),
+            doc=_(u'groups to remove'),
             alwaysask=True,
         ),
     )
@@ -1474,13 +1676,20 @@ class sudorule_remove_user(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
         parameters.Str(
             'user',
             required=False,
             multivalue=True,
             cli_name='users',
             label=_(u'member user'),
-            doc=_(u'comma-separated list of users to remove'),
+            doc=_(u'users to remove'),
             alwaysask=True,
         ),
         parameters.Str(
@@ -1489,7 +1698,7 @@ class sudorule_remove_user(Method):
             multivalue=True,
             cli_name='groups',
             label=_(u'member group'),
-            doc=_(u'comma-separated list of groups to remove'),
+            doc=_(u'groups to remove'),
             alwaysask=True,
         ),
     )
@@ -1543,6 +1752,13 @@ class sudorule_show(Method):
             default=False,
             autofill=True,
         ),
+        parameters.Flag(
+            'no_members',
+            doc=_(u'Suppress processing of membership attributes.'),
+            exclude=('webui', 'cli'),
+            default=False,
+            autofill=True,
+        ),
     )
     has_output = (
         output.Output(
@@ -1553,9 +1769,8 @@ class sudorule_show(Method):
         output.Entry(
             'result',
         ),
-        output.Output(
+        output.PrimaryKey(
             'value',
-            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
