@@ -217,6 +217,11 @@ class user(Object):
             multivalue=True,
             label=_(u'Car License'),
         ),
+        parameters.Bool(
+            'nsaccountlock',
+            required=False,
+            label=_(u'Account disabled'),
+        ),
         parameters.Str(
             'ipasshpubkey',
             required=False,
@@ -268,23 +273,6 @@ class user(Object):
             required=False,
             label=_(u'Preferred Language'),
         ),
-        parameters.Bytes(
-            'usercertificate',
-            required=False,
-            multivalue=True,
-            label=_(u'Certificate'),
-            doc=_(u'Base-64 encoded server certificate'),
-        ),
-        parameters.Bool(
-            'nsaccountlock',
-            required=False,
-            label=_(u'Account disabled'),
-        ),
-        parameters.Bool(
-            'preserved',
-            required=False,
-            label=_(u'Preserved user'),
-        ),
         parameters.Flag(
             'has_password',
             label=_(u'Password'),
@@ -355,7 +343,7 @@ class user_add(Method):
             'uid',
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
@@ -373,21 +361,21 @@ class user_add(Method):
         parameters.Str(
             'cn',
             label=_(u'Full name'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
             autofill=True,
         ),
         parameters.Str(
             'displayname',
             required=False,
             label=_(u'Display name'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
             autofill=True,
         ),
         parameters.Str(
             'initials',
             required=False,
             label=_(u'Initials'),
-            default_from=DefaultFrom(lambda givenname, sn: '%c%c' % (givenname[0], sn[0]), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%c%c' % (givenname[0], sn[0]), 'principal'),
             autofill=True,
         ),
         parameters.Str(
@@ -400,7 +388,7 @@ class user_add(Method):
             'gecos',
             required=False,
             label=_(u'GECOS'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
             autofill=True,
         ),
         parameters.Str(
@@ -414,7 +402,7 @@ class user_add(Method):
             required=False,
             cli_name='principal',
             label=_(u'Kerberos principal'),
-            default_from=DefaultFrom(lambda uid: '%s@%s' % (uid.lower(), api.env.realm), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda uid: '%s@%s' % (uid.lower(), api.env.realm), 'principal'),
             autofill=True,
             no_convert=True,
         ),
@@ -530,6 +518,12 @@ class user_add(Method):
             multivalue=True,
             label=_(u'Car License'),
         ),
+        parameters.Bool(
+            'nsaccountlock',
+            required=False,
+            label=_(u'Account disabled'),
+            exclude=('cli', 'webui'),
+        ),
         parameters.Str(
             'ipasshpubkey',
             required=False,
@@ -588,20 +582,6 @@ class user_add(Method):
             required=False,
             label=_(u'Preferred Language'),
         ),
-        parameters.Bytes(
-            'usercertificate',
-            required=False,
-            multivalue=True,
-            cli_name='certificate',
-            label=_(u'Certificate'),
-            doc=_(u'Base-64 encoded server certificate'),
-        ),
-        parameters.Bool(
-            'nsaccountlock',
-            required=False,
-            label=_(u'Account disabled'),
-            exclude=('cli', 'webui'),
-        ),
         parameters.Str(
             'setattr',
             required=False,
@@ -660,129 +640,6 @@ class user_add(Method):
     )
 
 
-@register()
-class user_add_cert(Method):
-    __doc__ = _("Add one or more certificates to the user entry")
-
-    takes_args = (
-        parameters.Str(
-            'uid',
-            cli_name='login',
-            label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
-            no_convert=True,
-        ),
-    )
-    takes_options = (
-        parameters.Flag(
-            'all',
-            doc=_(u'Retrieve and print all attributes from the server. Affects command output.'),
-            exclude=('webui',),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Flag(
-            'raw',
-            doc=_(u'Print entries as stored on the server. Only affects output format.'),
-            exclude=('webui',),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Bytes(
-            'usercertificate',
-            required=False,
-            multivalue=True,
-            cli_name='certificate',
-            label=_(u'Certificate'),
-            doc=_(u'Base-64 encoded server certificate'),
-            alwaysask=True,
-        ),
-    )
-    has_output = (
-        output.Output(
-            'summary',
-            (unicode, type(None)),
-            doc=_(u'User-friendly description of action performed'),
-        ),
-        output.Entry(
-            'result',
-        ),
-        output.PrimaryKey(
-            'value',
-            doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
-        ),
-    )
-
-
-@register()
-class user_add_manager(Method):
-    __doc__ = _("Add a manager to the user entry")
-
-    takes_args = (
-        parameters.Str(
-            'uid',
-            cli_name='login',
-            label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
-            no_convert=True,
-        ),
-    )
-    takes_options = (
-        parameters.Flag(
-            'all',
-            doc=_(u'Retrieve and print all attributes from the server. Affects command output.'),
-            exclude=('webui',),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Flag(
-            'raw',
-            doc=_(u'Print entries as stored on the server. Only affects output format.'),
-            exclude=('webui',),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Str(
-            'user',
-            required=False,
-            multivalue=True,
-            cli_name='users',
-            label=_(u'member user'),
-            doc=_(u'users to add'),
-            alwaysask=True,
-        ),
-    )
-    has_output = (
-        output.Entry(
-            'result',
-        ),
-        output.Output(
-            'failed',
-            dict,
-            doc=_(u'Members that could not be added'),
-        ),
-        output.Output(
-            'completed',
-            int,
-            doc=_(u'Number of members added'),
-        ),
-    )
-
-
 @register()
 class user_del(Method):
     __doc__ = _("Delete a user.")
@@ -793,7 +650,7 @@ class user_del(Method):
             multivalue=True,
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
@@ -804,11 +661,6 @@ class user_del(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Bool(
-            'preserve',
-            required=False,
-            exclude=('cli',),
-        ),
     )
     has_output = (
         output.Output(
@@ -836,7 +688,7 @@ class user_disable(Method):
             'uid',
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
@@ -869,7 +721,7 @@ class user_enable(Method):
             'uid',
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
@@ -910,7 +762,7 @@ class user_find(Method):
             required=False,
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
         parameters.Str(
@@ -929,19 +781,19 @@ class user_find(Method):
             'cn',
             required=False,
             label=_(u'Full name'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
         ),
         parameters.Str(
             'displayname',
             required=False,
             label=_(u'Display name'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
         ),
         parameters.Str(
             'initials',
             required=False,
             label=_(u'Initials'),
-            default_from=DefaultFrom(lambda givenname, sn: '%c%c' % (givenname[0], sn[0]), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%c%c' % (givenname[0], sn[0]), 'principal'),
         ),
         parameters.Str(
             'homedirectory',
@@ -953,7 +805,7 @@ class user_find(Method):
             'gecos',
             required=False,
             label=_(u'GECOS'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
         ),
         parameters.Str(
             'loginshell',
@@ -966,7 +818,7 @@ class user_find(Method):
             required=False,
             cli_name='principal',
             label=_(u'Kerberos principal'),
-            default_from=DefaultFrom(lambda uid: '%s@%s' % (uid.lower(), api.env.realm), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda uid: '%s@%s' % (uid.lower(), api.env.realm), 'principal'),
             no_convert=True,
         ),
         parameters.DateTime(
@@ -1074,6 +926,12 @@ class user_find(Method):
             multivalue=True,
             label=_(u'Car License'),
         ),
+        parameters.Bool(
+            'nsaccountlock',
+            required=False,
+            label=_(u'Account disabled'),
+            exclude=('cli', 'webui'),
+        ),
         parameters.Str(
             'ipauserauthtype',
             required=False,
@@ -1124,37 +982,17 @@ class user_find(Method):
             required=False,
             label=_(u'Preferred Language'),
         ),
-        parameters.Bytes(
-            'usercertificate',
-            required=False,
-            multivalue=True,
-            cli_name='certificate',
-            label=_(u'Certificate'),
-            doc=_(u'Base-64 encoded server certificate'),
-        ),
-        parameters.Bool(
-            'nsaccountlock',
-            required=False,
-            label=_(u'Account disabled'),
-            exclude=('cli', 'webui'),
-        ),
-        parameters.Bool(
-            'preserved',
-            required=False,
-            label=_(u'Preserved user'),
-            default=False,
-        ),
         parameters.Int(
             'timelimit',
             required=False,
             label=_(u'Time Limit'),
-            doc=_(u'Time limit of search in seconds (0 is unlimited)'),
+            doc=_(u'Time limit of search in seconds'),
         ),
         parameters.Int(
             'sizelimit',
             required=False,
             label=_(u'Size Limit'),
-            doc=_(u'Maximum number of entries returned (0 is unlimited)'),
+            doc=_(u'Maximum number of entries returned'),
         ),
         parameters.Flag(
             'whoami',
@@ -1304,7 +1142,7 @@ class user_mod(Method):
             'uid',
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
@@ -1325,19 +1163,19 @@ class user_mod(Method):
             'cn',
             required=False,
             label=_(u'Full name'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
         ),
         parameters.Str(
             'displayname',
             required=False,
             label=_(u'Display name'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
         ),
         parameters.Str(
             'initials',
             required=False,
             label=_(u'Initials'),
-            default_from=DefaultFrom(lambda givenname, sn: '%c%c' % (givenname[0], sn[0]), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%c%c' % (givenname[0], sn[0]), 'principal'),
         ),
         parameters.Str(
             'homedirectory',
@@ -1349,7 +1187,7 @@ class user_mod(Method):
             'gecos',
             required=False,
             label=_(u'GECOS'),
-            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: '%s %s' % (givenname, sn), 'principal'),
         ),
         parameters.Str(
             'loginshell',
@@ -1469,6 +1307,12 @@ class user_mod(Method):
             multivalue=True,
             label=_(u'Car License'),
         ),
+        parameters.Bool(
+            'nsaccountlock',
+            required=False,
+            label=_(u'Account disabled'),
+            exclude=('cli', 'webui'),
+        ),
         parameters.Str(
             'ipasshpubkey',
             required=False,
@@ -1527,20 +1371,6 @@ class user_mod(Method):
             required=False,
             label=_(u'Preferred Language'),
         ),
-        parameters.Bytes(
-            'usercertificate',
-            required=False,
-            multivalue=True,
-            cli_name='certificate',
-            label=_(u'Certificate'),
-            doc=_(u'Base-64 encoded server certificate'),
-        ),
-        parameters.Bool(
-            'nsaccountlock',
-            required=False,
-            label=_(u'Account disabled'),
-            exclude=('cli', 'webui'),
-        ),
         parameters.Str(
             'setattr',
             required=False,
@@ -1595,7 +1425,7 @@ class user_mod(Method):
             required=False,
             label=_(u'Rename'),
             doc=_(u'Rename the user object'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
@@ -1615,129 +1445,6 @@ class user_mod(Method):
     )
 
 
-@register()
-class user_remove_cert(Method):
-    __doc__ = _("Remove one or more certificates to the user entry")
-
-    takes_args = (
-        parameters.Str(
-            'uid',
-            cli_name='login',
-            label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
-            no_convert=True,
-        ),
-    )
-    takes_options = (
-        parameters.Flag(
-            'all',
-            doc=_(u'Retrieve and print all attributes from the server. Affects command output.'),
-            exclude=('webui',),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Flag(
-            'raw',
-            doc=_(u'Print entries as stored on the server. Only affects output format.'),
-            exclude=('webui',),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Bytes(
-            'usercertificate',
-            required=False,
-            multivalue=True,
-            cli_name='certificate',
-            label=_(u'Certificate'),
-            doc=_(u'Base-64 encoded server certificate'),
-            alwaysask=True,
-        ),
-    )
-    has_output = (
-        output.Output(
-            'summary',
-            (unicode, type(None)),
-            doc=_(u'User-friendly description of action performed'),
-        ),
-        output.Entry(
-            'result',
-        ),
-        output.PrimaryKey(
-            'value',
-            doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
-        ),
-    )
-
-
-@register()
-class user_remove_manager(Method):
-    __doc__ = _("Remove a manager to the user entry")
-
-    takes_args = (
-        parameters.Str(
-            'uid',
-            cli_name='login',
-            label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
-            no_convert=True,
-        ),
-    )
-    takes_options = (
-        parameters.Flag(
-            'all',
-            doc=_(u'Retrieve and print all attributes from the server. Affects command output.'),
-            exclude=('webui',),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Flag(
-            'raw',
-            doc=_(u'Print entries as stored on the server. Only affects output format.'),
-            exclude=('webui',),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Flag(
-            'no_members',
-            doc=_(u'Suppress processing of membership attributes.'),
-            exclude=('webui', 'cli'),
-            default=False,
-            autofill=True,
-        ),
-        parameters.Str(
-            'user',
-            required=False,
-            multivalue=True,
-            cli_name='users',
-            label=_(u'member user'),
-            doc=_(u'users to remove'),
-            alwaysask=True,
-        ),
-    )
-    has_output = (
-        output.Entry(
-            'result',
-        ),
-        output.Output(
-            'failed',
-            dict,
-            doc=_(u'Members that could not be removed'),
-        ),
-        output.Output(
-            'completed',
-            int,
-            doc=_(u'Number of members removed'),
-        ),
-    )
-
-
 @register()
 class user_show(Method):
     __doc__ = _("Display information about a user.")
@@ -1747,7 +1454,7 @@ class user_show(Method):
             'uid',
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
@@ -1759,11 +1466,6 @@ class user_show(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Str(
-            'out',
-            required=False,
-            doc=_(u'file to store certificate in'),
-        ),
         parameters.Flag(
             'all',
             doc=_(u'Retrieve and print all attributes from the server. Affects command output.'),
@@ -1802,45 +1504,6 @@ class user_show(Method):
     )
 
 
-@register()
-class user_stage(Method):
-    __doc__ = _("Move deleted user into staged area")
-
-    takes_args = (
-        parameters.Str(
-            'uid',
-            multivalue=True,
-            cli_name='login',
-            label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
-            no_convert=True,
-        ),
-    )
-    takes_options = (
-        parameters.Flag(
-            'continue',
-            doc=_(u"Continuous mode: Don't stop on errors."),
-            default=False,
-            autofill=True,
-        ),
-    )
-    has_output = (
-        output.Output(
-            'summary',
-            (unicode, type(None)),
-            doc=_(u'User-friendly description of action performed'),
-        ),
-        output.Output(
-            'result',
-            dict,
-            doc=_(u'List of deletions that failed'),
-        ),
-        output.ListOfPrimaryKeys(
-            'value',
-        ),
-    )
-
-
 @register()
 class user_status(Method):
     __doc__ = _("""
@@ -1871,7 +1534,7 @@ Lockout status of a user account
             'uid',
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
@@ -1920,39 +1583,6 @@ Lockout status of a user account
     )
 
 
-@register()
-class user_undel(Method):
-    __doc__ = _("Undelete a delete user account.")
-
-    takes_args = (
-        parameters.Str(
-            'uid',
-            cli_name='login',
-            label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
-            no_convert=True,
-        ),
-    )
-    takes_options = (
-    )
-    has_output = (
-        output.Output(
-            'summary',
-            (unicode, type(None)),
-            doc=_(u'User-friendly description of action performed'),
-        ),
-        output.Output(
-            'result',
-            bool,
-            doc=_(u'True means the operation was successful'),
-        ),
-        output.PrimaryKey(
-            'value',
-            doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
-        ),
-    )
-
-
 @register()
 class user_unlock(Method):
     __doc__ = _("""
@@ -1969,7 +1599,7 @@ Unlock a user account
             'uid',
             cli_name='login',
             label=_(u'User login'),
-            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'iparepltoposegmentleftnode', 'iparepltoposegmentrightnode'),
+            default_from=DefaultFrom(lambda givenname, sn: givenname[0] + sn, 'principal'),
             no_convert=True,
         ),
     )
