@@ -468,13 +468,13 @@ class group_find(Method):
             'timelimit',
             required=False,
             label=_(u'Time Limit'),
-            doc=_(u'Time limit of search in seconds'),
+            doc=_(u'Time limit of search in seconds (0 is unlimited)'),
         ),
         parameters.Int(
             'sizelimit',
             required=False,
             label=_(u'Size Limit'),
-            doc=_(u'Maximum number of entries returned'),
+            doc=_(u'Maximum number of entries returned (0 is unlimited)'),
         ),
         parameters.Flag(
             'private',
