@@ -40,7 +40,7 @@ Additionally an ID range of the local domain may set
 
 and an ID range of a trusted domain must set
  - rid-base: the first RID of the corresponding RID range
- - sid: domain SID of the trusted domain
+ - dom_sid: domain SID of the trusted domain
 
 
 
@@ -170,16 +170,10 @@ class idrange(Object):
             required=False,
             label=_(u'Domain SID of the trusted domain'),
         ),
-        parameters.Str(
-            'ipanttrusteddomainname',
-            required=False,
-            label=_(u'Name of the trusted domain'),
-        ),
         parameters.Str(
             'iparangetype',
             required=False,
             label=_(u'Range type'),
-            doc=_(u'ID range type, one of ipa-ad-trust-posix, ipa-ad-trust, ipa-local'),
         ),
     )
 
@@ -201,7 +195,7 @@ Add new ID range.
 
     may be given for a new ID range for the local domain while
 
-        --rid-base
+        --rid-bas
         --dom-sid
 
     must be given to add a new range for a trusted AD domain.
@@ -254,19 +248,11 @@ Add new ID range.
             cli_name='dom_sid',
             label=_(u'Domain SID of the trusted domain'),
         ),
-        parameters.Str(
-            'ipanttrusteddomainname',
-            required=False,
-            cli_name='dom_name',
-            label=_(u'Name of the trusted domain'),
-        ),
         parameters.Str(
             'iparangetype',
             required=False,
-            cli_name='type',
-            cli_metavar="['ipa-ad-trust-posix', 'ipa-ad-trust', 'ipa-local']",
             label=_(u'Range type'),
-            doc=_(u'ID range type, one of ipa-ad-trust-posix, ipa-ad-trust, ipa-local'),
+            exclude=('cli', 'webui'),
         ),
         parameters.Str(
             'setattr',
@@ -306,8 +292,9 @@ Add new ID range.
         output.Entry(
             'result',
         ),
-        output.PrimaryKey(
+        output.Output(
             'value',
+            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
@@ -344,8 +331,10 @@ class idrange_del(Method):
             dict,
             doc=_(u'List of deletions that failed'),
         ),
-        output.ListOfPrimaryKeys(
+        output.Output(
             'value',
+            unicode,
+            doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
 
@@ -401,10 +390,8 @@ class idrange_find(Method):
         parameters.Str(
             'iparangetype',
             required=False,
-            cli_name='type',
-            cli_metavar="['ipa-ad-trust-posix', 'ipa-ad-trust', 'ipa-local']",
             label=_(u'Range type'),
-            doc=_(u'ID range type, one of ipa-ad-trust-posix, ipa-ad-trust, ipa-local'),
+            exclude=('cli', 'webui'),
         ),
         parameters.Int(
             'timelimit',
@@ -499,6 +486,18 @@ class idrange_mod(Method):
             cli_name='secondary_rid_base',
             label=_(u'First RID of the secondary RID range'),
         ),
+        parameters.Str(
+            'ipanttrusteddomainsid',
+            required=False,
+            cli_name='dom_sid',
+            label=_(u'Domain SID of the trusted domain'),
+        ),
+        parameters.Str(
+            'iparangetype',
+            required=False,
+            label=_(u'Range type'),
+            exclude=('cli', 'webui'),
+        ),
         parameters.Str(
             'setattr',
             required=False,
@@ -527,18 +526,6 @@ class idrange_mod(Method):
             default=False,
             autofill=True,
         ),
-        parameters.Str(
-            'ipanttrusteddomainsid',
-            required=False,
-            deprecated=True,
-            exclude=('cli', 'webui'),
-        ),
-        parameters.Str(
-            'ipanttrusteddomainname',
-            required=False,
-            deprecated=True,
-            exclude=('cli', 'webui'),
-        ),
         parameters.Flag(
             'all',
             doc=_(u'Retrieve and print all attributes from the server. Affects command output.'),
@@ -563,8 +550,9 @@ class idrange_mod(Method):
         output.Entry(
             'result',
         ),
-        output.PrimaryKey(
+        output.Output(
             'value',
+            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
@@ -613,8 +601,9 @@ class idrange_show(Method):
         output.Entry(
             'result',
         ),
-        output.PrimaryKey(
+        output.Output(
             'value',
+            unicode,
             doc=_(u"The primary_key value of the entry, e.g. 'jdoe' for a user"),
         ),
     )
