@@ -23,6 +23,7 @@ Functionality for Command Line Interface.
 from __future__ import print_function
 
 import atexit
+import builtins
 import importlib
 import logging
 import textwrap
@@ -48,14 +49,11 @@ import six
 from six.moves import input
 
 from ipalib.util import (
-    check_client_configuration, get_terminal_height, open_in_pager
+    check_client_configuration, get_pager, get_terminal_height, open_in_pager
 )
 
 if six.PY3:
     unicode = str
-    import builtins  # pylint: disable=import-error
-else:
-    import __builtin__ as builtins  # pylint: disable=import-error
 
 if six.PY2:
     reload(sys)  # pylint: disable=reload-builtin, undefined-variable
@@ -70,7 +68,7 @@ from ipalib.errors import (PublicError, CommandError, HelpError, InternalError,
 from ipalib.constants import CLI_TAB, LDAP_GENERALIZED_TIME_FORMAT
 from ipalib.parameters import File, BinaryFile, Str, Enum, Any, Flag
 from ipalib.text import _
-from ipalib import api  # pylint: disable=unused-import
+from ipalib import api
 from ipapython.dnsutil import DNSName
 from ipapython.admintool import ScriptError
 
@@ -311,7 +309,7 @@ class textui(backend.Backend):
           objectClass: top
           objectClass: someClass
         """
-        assert isinstance(attr, six.string_types)
+        assert isinstance(attr, str)
         if not isinstance(value, (list, tuple)):
             # single-value attribute
             self.print_indented(format % (attr, self.encode_binary(value)), indent)
@@ -450,7 +448,7 @@ class textui(backend.Backend):
         ------------------
         Only dashed above.
         """
-        assert isinstance(dash, six.string_types)
+        assert isinstance(dash, str)
         assert len(dash) == 1
         dashes = dash * len(string)
         if above:
@@ -611,7 +609,7 @@ class textui(backend.Backend):
             prompt = u'%s Yes/No: ' % label
 
         while True:
-            data = self.prompt_helper(prompt, label).lower() #pylint: disable=E1103
+            data = self.prompt_helper(prompt, label).lower()
 
             if data in (u'yes', u'y'):
                 return True
@@ -682,9 +680,9 @@ class textui(backend.Backend):
             except EOFError:
                 return -2
 
-            if resp.lower() == "q": #pylint: disable=E1103
+            if resp.lower() == "q":
                 return -2
-            if resp.lower() == "a": #pylint: disable=E1103
+            if resp.lower() == "a":
                 return -1
             try:
                 selection = int(resp) - 1
@@ -704,7 +702,7 @@ class help(frontend.Local):
     """
     Display help for a command or topic.
     """
-    class Writer(object):
+    class Writer:
         """
         Writer abstraction
         """
@@ -723,9 +721,11 @@ class help(frontend.Local):
             self.buffer.append(unicode(string))
 
         def write(self):
-            if self.buffer_length > get_terminal_height():
+            pager = get_pager()
+
+            if pager and self.buffer_length > get_terminal_height():
                 data = "\n".join(self.buffer).encode("utf-8")
-                open_in_pager(data)
+                open_in_pager(data, pager)
             else:
                 try:
                     for line in self.buffer:
@@ -890,13 +890,13 @@ class help(frontend.Local):
                 commands = self._topics[topic][2]
             else:
                 commands = []
-                for t in self._topics:
-                    if type(self._topics[t][2]) is not dict:
+                for v in self._topics.values():
+                    if not isinstance(v[2], dict):
                         continue
-                    if topic not in self._topics[t][2]:
+                    if topic not in v[2]:
                         continue
-                    mcl = self._topics[t][2][topic][1]
-                    commands = self._topics[t][2][topic][2]
+                    mcl = v[2][topic][1]
+                    commands = v[2][topic][2]
                     break
 
             doc, _topic = self._get_topic(topic)
@@ -949,6 +949,25 @@ class show_mappings(frontend.Command):
             print(to_cli(item[0]).ljust(mcl)+' : '+item[1])
 
 
+class IPACompleter(rlcompleter.Completer):
+    def _callable_postfix(self, val, word):
+        # Don't add '(' postfix for callable API objects
+        if isinstance(val, (plugable.APINameSpace, plugable.API)):
+            return word
+        return super()._callable_postfix(val, word)
+
+
+class InteractiveConsole(code.InteractiveConsole):
+    def showtraceback(self):
+        ei = sys.exc_info()
+        e = ei[1]
+        if isinstance(e, PublicError):
+            self.write('IPA public error exception: %s: %s\n' %
+                       (e.__class__.__name__, str(e)))
+        else:
+            super().showtraceback()
+
+
 class console(frontend.Command):
     """Start the IPA interactive Python console, or run a script.
 
@@ -964,7 +983,7 @@ class console(frontend.Command):
     def _setup_tab_completion(self, local):
         readline.parse_and_bind("tab: complete")
         # completer with custom locals
-        readline.set_completer(rlcompleter.Completer(local).complete)
+        readline.set_completer(IPACompleter(local).complete)
         # load history
         history = os.path.join(api.env.dot_ipa, "console.history")
         try:
@@ -1010,13 +1029,14 @@ class console(frontend.Command):
         else:
             if readline is not None:
                 self._setup_tab_completion(local)
-            code.interact(
+            cons = InteractiveConsole(local)
+            cons.interact(
                 "\n".join((
                     "(Custom IPA interactive Python console)",
                     "    api: IPA API object",
                     "    pp: pretty printer",
                 )),
-                local=local
+                exitmsg=None
             )
 
 
@@ -1082,7 +1102,7 @@ cli_application_commands = (
 )
 
 
-class Collector(object):
+class Collector:
     def __init__(self):
         object.__setattr__(self, '_Collector__options', {})
 
@@ -1403,9 +1423,7 @@ class cli(backend.Executioner):
                 elif p.stdin_if_missing:
                     try:
                         if six.PY3 and p.type is bytes:
-                            # pylint: disable=no-member
                             raw = sys.stdin.buffer.read()
-                            # pylint: enable=no-member
                         else:
                             raw = sys.stdin.read()
                     except IOError as e:
@@ -1454,14 +1472,14 @@ def run(api):
         (_options, argv) = api.bootstrap_with_global_options(context='cli')
 
         try:
-            check_client_configuration()
+            check_client_configuration(env=api.env)
         except ScriptError as e:
             sys.exit(e)
 
         for klass in cli_plugins:
             api.add_plugin(klass)
         api.finalize()
-        if not 'config_loaded' in api.env and not 'help' in argv:
+        if 'config_loaded' not in api.env and 'help' not in argv:
             raise NotConfiguredError()
         sys.exit(api.Backend.cli.run(argv))
     except KeyboardInterrupt:
