@@ -22,16 +22,37 @@
 Per-request thread-local data.
 """
 
+import contextlib
 import threading
 
 from ipalib.base import ReadOnly, lock
-from ipalib.constants import OVERRIDE_ERROR, CALLABLE_ERROR
+from ipalib.constants import CALLABLE_ERROR
 
 
 # Thread-local storage of most per-request information
 context = threading.local()
 
 
+class _FrameContext:
+    pass
+
+
+@contextlib.contextmanager
+def context_frame():
+    try:
+        frame_back = context.current_frame
+    except AttributeError:
+        pass
+    context.current_frame = _FrameContext()
+    try:
+        yield
+    finally:
+        try:
+            context.current_frame = frame_back
+        except UnboundLocalError:
+            del context.current_frame
+
+
 class Connection(ReadOnly):
     """
     Base class for connection objects stored on `request.context`.
@@ -56,4 +77,3 @@ def destroy_context():
         if isinstance(value, Connection):
             value.disconnect()
     context.__dict__.clear()
-
