@@ -60,11 +60,11 @@ from ipalib.constants import (
     TLS_VERSIONS, TLS_VERSION_MINIMAL, TLS_VERSION_MAXIMAL,
     TLS_VERSION_DEFAULT_MIN, TLS_VERSION_DEFAULT_MAX,
 )
-# pylint: disable=ipa-forbidden-import
 from ipalib.facts import is_ipa_client_configured
 from ipalib.text import _
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
+from ipapython import ipautil
 from ipapython.ssh import SSHPublicKey
 from ipapython.dn import DN, RDN
 from ipapython.dnsutil import (
@@ -74,9 +74,10 @@ from ipapython.dnsutil import (
     resolve_ip_addresses,
 )
 from ipapython.admintool import ScriptError
+from ipapython.kerberos import Principal
 
 if sys.version_info >= (3, 2):
-    import reprlib  # pylint: disable=import-error
+    import reprlib
 else:
     reprlib = None
 
@@ -157,7 +158,7 @@ def isvalid_base64(data):
 
     The character set must only include of a-z, A-Z, 0-9, + or / and
     be padded with = to be a length divisible by 4 (so only 0-2 =s are
-    allowed). Its length must be divisible by 4. White space is
+    allowed). Its length must be divisible by 4. Whitespace is
     not significant so it is removed.
 
     This doesn't guarantee we have a base64-encoded value, just that it
@@ -242,7 +243,7 @@ def normalize_zone(zone):
 def get_proper_tls_version_span(tls_version_min, tls_version_max):
     """
     This function checks whether the given TLS versions are known in
-    FreeIPA and that these versions fulfill the requirements for minimal
+    IPA and that these versions fulfill the requirements for minimal
     TLS version (see
     `ipalib.constants: TLS_VERSIONS, TLS_VERSION_MINIMAL`).
 
@@ -320,7 +321,6 @@ def create_https_connection(
             is not encrypted.
     :returns An established HTTPS connection to host:port
     """
-    # pylint: disable=no-member
     tls_cutoff_map = {
         "ssl2": ssl.OP_NO_SSLv2,
         "ssl3": ssl.OP_NO_SSLv3,
@@ -329,7 +329,6 @@ def create_https_connection(
         "tls1.2": ssl.OP_NO_TLSv1_2,
         "tls1.3": getattr(ssl, "OP_NO_TLSv1_3", 0),
     }
-    # pylint: enable=no-member
 
     if cafile is None:
         raise RuntimeError("cafile argument is required to perform server "
@@ -341,9 +340,10 @@ def create_https_connection(
 
     # official Python documentation states that the best option to get
     # TLSv1 and later is to setup SSLContext with PROTOCOL_SSLv23
-    # and then negate the insecure SSLv2 and SSLv3
-    # pylint: disable=no-member
-    ctx = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
+    # and then negate the insecure SSLv2 and SSLv3. However, with Python 3.10
+    # PROTOCOL_SSLv23 is deprecated as well as PROTOCOL_TLS. We should use
+    # PROTOCOL_TLS_CLIENT since Python 3.6
+    ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
     ctx.options |= (
         ssl.OP_ALL | ssl.OP_NO_COMPRESSION | ssl.OP_SINGLE_DH_USE |
         ssl.OP_SINGLE_ECDH_USE
@@ -356,7 +356,6 @@ def create_https_connection(
     # remove the slice of negating protocol options according to options
     tls_span = get_proper_tls_version_span(tls_version_min, tls_version_max)
 
-    # pylint: enable=no-member
     # set up the correct TLS version flags for the SSL context
     if tls_span is not None:
         for version in TLS_VERSIONS:
@@ -1029,6 +1028,31 @@ def has_managed_topology(api):
     return domainlevel > DOMAIN_LEVEL_0
 
 
+def print_replication_status(entry, verbose):
+    """Pretty print nsds5replicalastinitstatus, nsds5replicalastinitend,
+    nsds5replicalastupdatestatus, nsds5replicalastupdateend for a
+    replication agreement.
+    """
+
+    if verbose:
+        initstatus = entry.single_value.get('nsds5replicalastinitstatus')
+        if initstatus is not None:
+            print("  last init status: %s" % initstatus)
+            print("  last init ended: %s" % str(
+                ipautil.parse_generalized_time(
+                    entry.single_value['nsds5replicalastinitend'])))
+        updatestatus = entry.single_value.get(
+            'nsds5replicalastupdatestatus'
+        )
+        if updatestatus is not None:
+            print("  last update status: %s" % updatestatus)
+            print("  last update ended: %s" % str(
+                ipautil.parse_generalized_time(
+                    entry.single_value['nsds5replicalastupdateend']
+                ))
+            )
+
+
 class classproperty:
     __slots__ = ('__doc__', 'fget')
 
@@ -1059,7 +1083,7 @@ class classproperty:
 class classobjectproperty(classproperty):
     # A class property that also passes the object to the getter
     # obj is None for class objects and 'self' for instance objects.
-    __slots__ = ('__doc__',)
+    __slots__ = ('__doc__',)  # pylint: disable=redefined-slots-in-subclass
 
     def __get__(self, obj, obj_type):
         if self.fget is not None:
@@ -1209,16 +1233,15 @@ def check_client_configuration(env=None):
         )
 
 
-def check_principal_realm_in_trust_namespace(api_instance, *keys):
+def _collect_trust_namespaces(api_instance, add_local=False):
     """
-    Check that principal name's suffix does not overlap with UPNs and realm
-    names of trusted forests.
+    Return UPNs and realm names of trusted forests.
 
     :param api_instance: API instance
-    :param suffixes: principal suffixes
+    :param add_local: bool flag
 
-    :raises: ValidationError if the suffix coincides with realm name, UPN
-    suffix or netbios name of trusted domains
+    :return: set of namespace names as strings.
+             If add_local is True, add own realm namesapce
     """
     trust_objects = api_instance.Command.trust_find(u'', sizelimit=0)['result']
 
@@ -1235,17 +1258,69 @@ def check_principal_realm_in_trust_namespace(api_instance, *keys):
 
         trust_suffix_namespace.add(obj['cn'][0].lower())
 
-    for principal in keys[-1]:
+    if add_local:
+        trust_suffix_namespace.add(api_instance.env.realm.lower())
+
+    return trust_suffix_namespace
+
+
+def check_principal_realm_in_trust_namespace(api_instance, *suffixes,
+                                             attr_name='krbprincipalname'):
+    """
+    Check that principal name's suffix does not overlap with UPNs and realm
+    names of trusted forests.
+
+    :param api_instance: API instance
+    :param suffixes: principal suffixes
+
+    :raises: ValidationError if the suffix coincides with realm name, UPN
+    suffix or netbios name of trusted domains
+    """
+    trust_suffix_namespace = _collect_trust_namespaces(api_instance,
+                                                       add_local=False)
+
+    for p in suffixes[-1]:
+        principal = Principal(p, realm=api_instance.env.realm)
         realm = principal.realm
         upn = principal.upn_suffix if principal.is_enterprise else None
 
         if realm in trust_suffix_namespace or upn in trust_suffix_namespace:
             raise errors.ValidationError(
-                name='krbprincipalname',
+                name=attr_name,
                 error=_('realm or UPN suffix overlaps with trusted domain '
                         'namespace'))
 
 
+def check_principal_realm_supported(api_instance, *suffixes,
+                                    attr_name='krbprincipalname'):
+    """
+    Check that principal name's suffix does not overlap with UPNs and realm
+    names of trusted forests.
+
+    :param api_instance: API instance
+    :param suffixes: principal suffixes
+
+    :raises: ValidationError if the suffix does not match with realm name, UPN
+    suffix or netbios name of trusted domains or IPA domain
+    """
+    trust_suffix_namespace = _collect_trust_namespaces(api_instance,
+                                                       add_local=True)
+
+    for p in suffixes[-1]:
+        principal = Principal(p, realm=api_instance.env.realm)
+        realm = principal.realm
+        upn = principal.upn_suffix if principal.is_enterprise else None
+
+        conditions = [(realm.lower() not in trust_suffix_namespace),
+                      (upn is not None and (
+                          upn.lower() not in trust_suffix_namespace))]
+        if any(conditions):
+            raise errors.ValidationError(
+                name=attr_name,
+                error=_('realm or UPN suffix outside of supported realm '
+                        'domains or trusted domains namespace'))
+
+
 def no_matching_interface_for_ip_address_warning(addr_list):
     for ip in addr_list:
         if not ip.get_matching_interface():
