@@ -9,7 +9,9 @@ This Fedora base platform module exports platform related constants.
 # Fallback to default constant definitions
 from __future__ import absolute_import
 
-from ipaplatform.redhat.constants import RedHatConstantsNamespace
+from ipaplatform.redhat.constants import (
+    RedHatConstantsNamespace, User, Group
+)
 from ipaplatform.osinfo import osinfo
 
 # Fedora 28 and earlier use /etc/sysconfig/nfs
@@ -18,6 +20,9 @@ from ipaplatform.osinfo import osinfo
 HAS_NFS_CONF = osinfo.version_number >= (30,)
 
 
+__all__ = ("constants", "User", "Group")
+
+
 class FedoraConstantsNamespace(RedHatConstantsNamespace):
     # Fedora allows installation of Python 2 and 3 mod_wsgi, but the modules
     # can't coexist. For Apache to load correct module.
@@ -27,4 +32,6 @@ class FedoraConstantsNamespace(RedHatConstantsNamespace):
     if HAS_NFS_CONF:
         SECURE_NFS_VAR = None
 
+    NAMED_OPENSSL_ENGINE = "pkcs11"
+
 constants = FedoraConstantsNamespace()
