@@ -28,6 +28,7 @@ import os
 import traceback
 from optparse import OptionGroup  # pylint: disable=deprecated-module
 
+from ipaplatform.osinfo import osinfo
 from ipapython import version
 from ipapython import config
 from ipapython.ipa_log_manager import standard_logging_setup
@@ -179,9 +180,8 @@ class AdminTool:
             return_value = self.run()
         except BaseException as exception:
             if isinstance(exception, ScriptError):
-                # pylint: disable=no-member
                 if exception.rval and exception.rval > return_value:
-                    return_value = exception.rval  # pylint: disable=no-member
+                    return_value = exception.rval
             traceback = sys.exc_info()[2]
             error_message, return_value = self.handle_error(exception)
             if return_value:
@@ -243,7 +243,7 @@ class AdminTool:
         root_logger = logging.getLogger()
         for handler in root_logger.handlers:
             if (isinstance(handler, logging.StreamHandler) and
-                    handler.stream is sys.stderr):  # pylint: disable=no-member
+                    handler.stream is sys.stderr):
                 root_logger.removeHandler(handler)
                 break
 
@@ -304,6 +304,8 @@ class AdminTool:
         logger.debug('%s was invoked with arguments %s and options: %s',
                      self.command_name, self.args, self.safe_options)
         logger.debug('IPA version %s', version.VENDOR_VERSION)
+        logger.debug('IPA platform %s', osinfo.platform)
+        logger.debug('IPA os-release %s %s', osinfo.name, osinfo.version)
 
     def log_failure(self, error_message, return_value, exception, backtrace):
         logger.debug('%s', ''.join(traceback.format_tb(backtrace)))
