@@ -471,12 +471,14 @@ def loadLibrary(module):
         # pylint: disable=no-member
         pDynLib = _ffi.dlopen(module, _ffi.RTLD_NOW | _ffi.RTLD_LOCAL)
     else:
-        raise Exception()
+        raise Error("PKCS#11 module name is empty")
 
     # Retrieve the entry point for C_GetFunctionList
     pGetFunctionList = pDynLib.C_GetFunctionList
     if pGetFunctionList == NULL:
-        raise Exception()
+        raise Error(
+            f"Module '{module}' has no function 'C_GetFunctionList'."
+        )
 
     # Store the handle so we can dlclose it later
 
@@ -521,23 +523,20 @@ CONST_RSA_PKCS_OAEP_PARAMS_ptr = new_ptr(CK_RSA_PKCS_OAEP_PARAMS, dict(
 #
 class P11HelperException(Exception):
     """parent class for all exceptions"""
-    pass
+
 P11HelperException.__name__ = 'Exception'
 
 
 class Error(P11HelperException):
     """general error"""
-    pass
 
 
 class NotFound(P11HelperException):
     """key not found"""
-    pass
 
 
 class DuplicationError(P11HelperException):
     """key already exists"""
-    pass
 
 
 ########################################################################
@@ -705,7 +704,7 @@ def _set_wrapping_mech_parameters(mech_type, mech):
 ########################################################################
 # P11_Helper object
 #
-class P11_Helper(object):
+class P11_Helper:
     @property
     def p11(self):
         return self.p11_ptr[0]
@@ -846,7 +845,7 @@ class P11_Helper(object):
         try:
             pGetFunctionList, module_handle = loadLibrary(library_path)
         except Exception:
-            raise Error("Could not load the library.")
+            raise Error(f"Could not load the library '{library_path}'.")
 
         self.module_handle = module_handle
 
@@ -914,7 +913,7 @@ class P11_Helper(object):
             check_return_value(rv, 'get token info')
 
             # softhsm always returns label 32 bytes long with padding made of
-            # white spaces (#32), so we have to rstrip() padding and compare
+            # whitespaces (#32), so we have to rstrip() padding and compare
             # Label was created by softhsm-util so it is not our fault that
             # there are #32 as padding (cffi initializes structures with
             # zeroes)
