@@ -426,7 +426,7 @@ class certmaprule_enable(LDAPQuery):
         except errors.NotFound:
             raise self.obj.handle_not_found(cn)
 
-        entry_attrs['ipaenabledflag'] = ['TRUE']
+        entry_attrs['ipaenabledflag'] = [True]
 
         try:
             ldap.update_entry(entry_attrs)
@@ -455,7 +455,7 @@ class certmaprule_disable(LDAPQuery):
         except errors.NotFound:
             raise self.obj.handle_not_found(cn)
 
-        entry_attrs['ipaenabledflag'] = ['FALSE']
+        entry_attrs['ipaenabledflag'] = [False]
 
         try:
             ldap.update_entry(entry_attrs)
@@ -614,7 +614,7 @@ class certmap_match(Search):
         The search is performed using SSSD's DBus interface
         Users.ListByCertificate.
         SSSD does the lookup based on certificate mapping rules, using
-        FreeIPA domain and trusted domains.
+        IPA domain and trusted domains.
         :raise RemoteRetrieveError: if DBus returns an exception
         """
         sssd = _sssd()
