@@ -689,7 +689,7 @@ class ra(rabase.rabase, RestClient):
         :param err_msg:   diagnostic error message, if not supplied it will be
                           'Unable to communicate with CMS'
         :param detail:    extra information that will be appended to err_msg
-                          inside a parenthesis
+                          inside a parenthesis. This may be an HTTP status msg
 
         Raise a CertificateOperationError and log the error message.
         """
@@ -701,6 +701,8 @@ class ra(rabase.rabase, RestClient):
             err_msg = u'%s (%s)' % (err_msg, detail)
 
         logger.error('%s.%s(): %s', type(self).__name__, func_name, err_msg)
+        if detail == 404:
+            raise errors.NotFound(reason=err_msg)
         raise errors.CertificateOperationError(error=err_msg)
 
     def _request(self, url, port, **kw):
@@ -1057,6 +1059,39 @@ class ra(rabase.rabase, RestClient):
 
         return cmd_result
 
+    def get_pki_version(self):
+        """
+        Retrieve the version of a remote PKI server.
+
+        The REST API request is a GET to the info URI:
+            GET /pki/rest/info HTTP/1.1
+
+        The response is: {"Version":"11.5.0","Attributes":{"Attribute":[]}}
+        """
+        path = "/pki/rest/info"
+        logger.debug('%s.get_pki_version()', type(self).__name__)
+        http_status, _http_headers, http_body = self._ssldo(
+            'GET', path,
+            headers={
+                'Content-Type': 'application/json',
+                'Accept': 'application/json',
+            },
+            use_session=False,
+        )
+        if http_status != 200:
+            self.raise_certificate_operation_error('get_pki_version',
+                                                   detail=http_status)
+
+        try:
+            response = json.loads(ipautil.decode_json(http_body))
+        except ValueError as e:
+            logger.debug("Response from CA was not valid JSON: %s", e)
+            raise errors.RemoteRetrieveError(
+                reason=_("Response from CA was not valid JSON")
+            )
+
+        return response.get('Version')
+
 
     def revoke_certificate(self, serial_number, revocation_reason=0):
         """
@@ -1123,6 +1158,20 @@ class ra(rabase.rabase, RestClient):
                 detail='7 is not a valid revocation reason'
             )
 
+        # dogtag changed the argument case for revocation from
+        # "reason" to "Reason" in PKI 11.4.0. Detect that change
+        # based on the remote version and pass the expected value
+        # in.
+        pki_version = pki.util.Version(self.get_pki_version())
+        if pki_version is None:
+            self.raise_certificate_operation_error('revoke_certificate',
+                                                   detail="Remove version not "
+                                                          "detected")
+        if pki_version < pki.util.Version("11.4.0"):
+            reason = "reason"
+        else:
+            reason = "Reason"
+
         # Convert serial number to integral type from string to properly handle
         # radix issues. Note: the int object constructor will properly handle
         # large magnitude integral values by returning a Python long type
@@ -1130,7 +1179,7 @@ class ra(rabase.rabase, RestClient):
         serial_number = int(serial_number, 0)
 
         path = 'agent/certs/{}/revoke'.format(serial_number)
-        data = '{{"reason":"{}"}}'.format(reasons[revocation_reason])
+        data = '{{"{}":"{}"}}'.format(reason, reasons[revocation_reason])
 
         http_status, _http_headers, http_body = self._ssldo(
             'POST', path,
@@ -1428,14 +1477,14 @@ class ra(rabase.rabase, RestClient):
             if issuer_dn:
                 response_request['issuer'] = issuer_dn
 
-            not_valid_before = cert.get('NotValidBefore')
-            if not_valid_before:
+            not_valid_before_utc = cert.get('NotValidBefore')
+            if not_valid_before_utc:
                 response_request['valid_not_before'] = (
-                    not_valid_before)
+                    not_valid_before_utc)
 
-            not_valid_after = cert.get('NotValidAfter')
-            if not_valid_after:
-                response_request['valid_not_after'] = (not_valid_after)
+            not_valid_after_utc = cert.get('NotValidAfter')
+            if not_valid_after_utc:
+                response_request['valid_not_after'] = (not_valid_after_utc)
 
             status = cert.get('Status')
             if status:
