@@ -21,24 +21,94 @@
 
 from __future__ import print_function, absolute_import
 
+from pprint import pformat
+
 import logging
 import os
 import tempfile
 import shutil
 import re
+import functools
 
 import pytest
 from pytest_multihost import make_multihost_fixture
 
 from ipapython import ipautil
 from ipaplatform.paths import paths
-from ipatests.test_util import yield_fixture
+from ipaplatform.constants import constants
+from . import fips
 from .config import Config
 from .env_config import get_global_config
 from . import tasks
 
 logger = logging.getLogger(__name__)
 
+CLASS_LOGFILES = [
+    # BIND logs
+    os.path.join(paths.NAMED_VAR_DIR, constants.NAMED_DATA_DIR),
+    # dirsrv logs
+    paths.VAR_LOG_DIRSRV,
+    # IPA install logs
+    paths.IPASERVER_INSTALL_LOG,
+    paths.IPASERVER_ADTRUST_INSTALL_LOG,
+    paths.IPASERVER_DNS_INSTALL_LOG,
+    paths.IPASERVER_KRA_INSTALL_LOG,
+    paths.IPACLIENT_INSTALL_LOG,
+    paths.IPAREPLICA_INSTALL_LOG,
+    paths.IPAREPLICA_CONNCHECK_LOG,
+    paths.IPAREPLICA_CA_INSTALL_LOG,
+    paths.IPA_CUSTODIA_AUDIT_LOG,
+    paths.IPACLIENTSAMBA_INSTALL_LOG,
+    paths.IPACLIENTSAMBA_UNINSTALL_LOG,
+    paths.IPATRUSTENABLEAGENT_LOG,
+    # IPA uninstall logs
+    paths.IPASERVER_UNINSTALL_LOG,
+    paths.IPACLIENT_UNINSTALL_LOG,
+    # IPA upgrade logs
+    paths.IPAUPGRADE_LOG,
+    # IPA backup and restore logs
+    paths.IPARESTORE_LOG,
+    paths.IPABACKUP_LOG,
+    # EPN log
+    paths.IPAEPN_LOG,
+    # kerberos related logs
+    paths.KADMIND_LOG,
+    paths.KRB5KDC_LOG,
+    # httpd logs
+    paths.VAR_LOG_HTTPD_DIR,
+    # dogtag logs
+    paths.VAR_LOG_PKI_DIR,
+    # dogtag conf
+    paths.PKI_TOMCAT_SERVER_XML,
+    paths.PKI_TOMCAT + "/ca/CS.cfg",
+    paths.PKI_TOMCAT + "/kra/CS.cfg",
+    paths.PKI_TOMCAT_ALIAS_DIR,
+    paths.PKI_TOMCAT_ALIAS_PWDFILE_TXT,
+    # selinux logs
+    paths.VAR_LOG_AUDIT,
+    # sssd
+    paths.VAR_LOG_SSSD_DIR,
+    # system
+    paths.RESOLV_CONF,
+    paths.HOSTS,
+    # IPA renewal lock
+    paths.IPA_RENEWAL_LOCK,
+    paths.LETS_ENCRYPT_LOG,
+    # resolvers management
+    paths.NETWORK_MANAGER_CONFIG,
+    paths.NETWORK_MANAGER_CONFIG_DIR,
+    paths.SYSTEMD_RESOLVED_CONF,
+    paths.SYSTEMD_RESOLVED_CONF_DIR,
+    '/var/log/samba',
+]
+
+
+def make_class_logs(host):
+    logs = list(CLASS_LOGFILES)
+    env_filename = os.path.join(host.config.test_dir, 'env.sh')
+    logs.append(env_filename)
+    return logs
+
 
 def pytest_addoption(parser):
     group = parser.getgroup("IPA integration tests")
@@ -56,33 +126,39 @@ def _get_logname_from_node(node):
     return name
 
 
-def collect_test_logs(node, logs_dict, test_config):
+def collect_test_logs(node, logs_dict, test_config, suffix=''):
     """Collect logs from a test
 
-    Calls collect_logs
+    Calls collect_logs and collect_systemd_journal
 
     :param node: The pytest collection node (request.node)
     :param logs_dict: Mapping of host to list of log filnames to collect
     :param test_config: Pytest configuration
+    :param suffix: The custom suffix of the name of logfiles' directory
     """
+    name = '{node}{suffix}'.format(
+        node=_get_logname_from_node(node),
+        suffix=suffix,
+    )
+    logfile_dir = test_config.getoption('logfile_dir')
     collect_logs(
-        name=_get_logname_from_node(node),
+        name=name,
         logs_dict=logs_dict,
-        logfile_dir=test_config.getoption('logfile_dir'),
+        logfile_dir=logfile_dir,
         beakerlib_plugin=test_config.pluginmanager.getplugin('BeakerLibPlugin'),
     )
 
+    hosts = logs_dict.keys()  # pylint: disable=dict-keys-not-iterating
+    collect_systemd_journal(name, hosts, logfile_dir)
 
-def collect_systemd_journal(node, hosts, test_config):
+
+def collect_systemd_journal(name, hosts, logfile_dir=None):
     """Collect systemd journal from remote hosts
 
-    :param node: The pytest collection node (request.node)
+    :param name: Name under which logs are collected, e.g. name of the test
     :param hosts: List of hosts from which to collect journal
-    :param test_config: Pytest configuration
+    :param logfile_dir: Directory to log to
     """
-    name = _get_logname_from_node(node)
-    logfile_dir = test_config.getoption('logfile_dir')
-
     if logfile_dir is None:
         return
 
@@ -132,6 +208,8 @@ def collect_logs(name, logs_dict, logfile_dir=None, beakerlib_plugin=None):
 
         for host, logs in logs_dict.items():
             logger.info('Collecting logs from: %s', host.hostname)
+            # make list of unique log filenames
+            logs = list(set(logs))
             dirname = os.path.join(topdirname, host.hostname)
             if not os.path.isdir(dirname):
                 os.makedirs(dirname)
@@ -141,8 +219,17 @@ def collect_logs(name, logs_dict, logfile_dir=None, beakerlib_plugin=None):
             tmpname = cmd.stdout_text.strip()
             # Tar up the logs on the remote server
             cmd = host.run_command(
-                ['tar', 'cJvf', tmpname, '--ignore-failed-read'] + logs,
-                log_stdout=False, raiseonerr=False)
+                [
+                    "tar",
+                    "cJvf",
+                    tmpname,
+                    "--ignore-failed-read",
+                    "--warning=no-failed-read",
+                    "--dereference",
+                ] + logs,
+                log_stdout=False,
+                raiseonerr=False,
+            )
             if cmd.returncode:
                 logger.warning('Could not collect all requested logs')
             # fetch tar file
@@ -181,23 +268,119 @@ def collect_logs(name, logs_dict, logfile_dir=None, beakerlib_plugin=None):
                 shutil.rmtree(topdirname)
 
 
+class IntegrationLogs:
+    """Represent logfile collections
+    Collection is a mapping of IPA hosts and a list of logfiles to be
+    collected. There are two types of collections: class and method.
+    The former contains a list of logfiles which will be collected on
+    each test (within class) completion, while the latter contains
+    a list of logfiles which will be collected on only certain test
+    completion (once).
+    """
+    def __init__(self):
+        self._class_logs = {}
+        self._method_logs = {}
+
+    def set_logs(self, host, logs):
+        self._class_logs[host] = list(logs)
+
+    @property
+    def method_logs(self):
+        return self._method_logs
+
+    @property
+    def class_logs(self):
+        return self._class_logs
+
+    def init_method_logs(self):
+        """Initilize method logs with the class ones"""
+        self._method_logs = {}
+        for host, logs in self._class_logs.items():
+            self._method_logs[host] = list(logs)
+
+    def collect_class_log(self, host, filename):
+        """Add class scope log
+        The file with the given filename will be collected from the
+        host on an each test completion(within a test class).
+        """
+        logger.info('Adding %s:%s to list of class logs to collect',
+                    host.external_hostname, filename)
+        self._class_logs.setdefault(host, []).append(filename)
+        self._method_logs.setdefault(host, []).append(filename)
+
+    def collect_method_log(self, host, filename):
+        """Add method scope log
+        The file with the given filename will be collected from the
+        host on a test completion.
+        """
+        logger.info('Adding %s:%s to list of method logs to collect',
+                    host.external_hostname, filename)
+        self._method_logs.setdefault(host, []).append(filename)
+
+
 @pytest.fixture(scope='class')
-def class_integration_logs():
-    """Internal fixture providing class-level logs_dict"""
-    return {}
+def class_integration_logs(request):
+    """Internal fixture providing class-level logs_dict
+    For adjusting collection of logs, please, use 'integration_logs'
+    fixture.
+    """
+    integration_logs = IntegrationLogs()
+    yield integration_logs
+    # since the main fixture of integration tests('mh') depends on
+    # this one the class logs collecting happens *after* the teardown
+    # of that fixture. The 'uninstall' is among the finalizers of 'mh'.
+    # This means that the logs collected here are the IPA *uninstall*
+    # logs.
+    class_logs = integration_logs.class_logs
+    collect_test_logs(request.node, class_logs, request.config,
+                      suffix='-uninstall')
 
 
-@yield_fixture
+@pytest.fixture
 def integration_logs(class_integration_logs, request):
     """Provides access to test integration logs, and collects after each test
+    To collect a logfile on a test completion one should add the dependency on
+    this fixture and call its 'collect_method_log' method.
+    For example, run TestFoo.
+    ```
+    class TestFoo(IntegrationTest):
+        def test_foo(self):
+            pass
+
+        def test_bar(self, integration_logs):
+            integration_logs.collect_method_log(self.master, '/logfile')
+    ```
+    '/logfile' will be collected only for 'test_bar' test.
+
+    To collect a logfile on a test class completion one should add the
+    dependency on this fixture and call its 'collect_class_log' method.
+    For example, run TestFoo.
+    ```
+    class TestFoo(IntegrationTest):
+        def test_foo(self, integration_logs):
+            integration_logs.collect_class_log(self.master, '/logfile')
+
+        def test_bar(self):
+            pass
+    ```
+    '/logfile' will be collected 3 times:
+    1) on 'test_foo' completion
+    2) on 'test_bar' completion
+    3) on 'TestFoo' completion
+
+    Note, the registration of a collection works at the runtime. This means
+    that if the '/logfile' will be registered in 'test_bar' then
+    it will not be collected on 'test_foo' completion:
+    1) on 'test_bar' completion
+    2) on 'TestFoo' completion
     """
+    class_integration_logs.init_method_logs()
     yield class_integration_logs
-    hosts = class_integration_logs.keys()
-    collect_test_logs(request.node, class_integration_logs, request.config)
-    collect_systemd_journal(request.node, hosts, request.config)
+    method_logs = class_integration_logs.method_logs
+    collect_test_logs(request.node, method_logs, request.config)
 
 
-@yield_fixture(scope='class')
+@pytest.fixture(scope='class')
 def mh(request, class_integration_logs):
     """IPA's multihost fixture object
     """
@@ -254,16 +437,15 @@ def mh(request, class_integration_logs):
         for domain in ad_domains:
             mh.ad_treedomains.extend(domain.hosts_by_role('ad_treedomain'))
 
-    cls.logs_to_collect = class_integration_logs
+    cls.logs_to_collect = class_integration_logs.class_logs
 
-    def collect_log(host, filename):
-        logger.info('Adding %s:%s to list of logs to collect',
-                    host.external_hostname, filename)
-        class_integration_logs.setdefault(host, []).append(filename)
+    if logger.isEnabledFor(logging.INFO):
+        logger.info(pformat(mh.config.to_dict()))
+
+    for ipa_host in mh.config.get_all_ipa_hosts():
+        class_integration_logs.set_logs(ipa_host, make_class_logs(ipa_host))
 
-    print(mh.config)
     for host in mh.config.get_all_hosts():
-        host.add_log_collector(collect_log)
         logger.info('Preparing host %s', host.hostname)
         tasks.prepare_host(host)
 
@@ -276,13 +458,17 @@ def mh(request, class_integration_logs):
     try:
         yield mh.install()
     finally:
-        hosts = list(cls.get_all_hosts())
-        for host in hosts:
-            host.remove_log_collector(collect_log)
-        collect_test_logs(
-            request.node, class_integration_logs, request.config
-        )
-        collect_systemd_journal(request.node, hosts, request.config)
+        # the 'mh' fixture depends on 'class_integration_logs' one,
+        # thus, the class logs collecting happens *after* the teardown
+        # of 'mh' fixture. The 'uninstall' is among the finalizers of 'mh'.
+        # This means that the logs collected here are the IPA *uninstall*
+        # logs and the 'install' ones can be removed during the IPA
+        # uninstall phase. To address this problem(e.g. installation error)
+        # the install logs will be collected into '{nodeid}-install' directory
+        # while the uninstall ones into '{nodeid}-uninstall'.
+        class_logs = class_integration_logs.class_logs
+        collect_test_logs(request.node, class_logs, request.config,
+                          suffix='-install')
 
 
 def add_compat_attrs(cls, mh):
@@ -317,3 +503,18 @@ def del_compat_attrs(cls):
         del cls.ad_subdomains
         del cls.ad_treedomains
     del cls.ad_domains
+
+
+def skip_if_fips(reason='Not supported in FIPS mode', host='master'):
+    if callable(reason):
+        raise TypeError('Invalid decorator usage, add "()"')
+
+    def decorator(test_method):
+        @functools.wraps(test_method)
+        def wrapper(instance, *args, **kwargs):
+            if fips.is_fips_enabled(getattr(instance, host)):
+                pytest.skip(reason)
+            else:
+                test_method(instance, *args, **kwargs)
+        return wrapper
+    return decorator
