@@ -26,9 +26,8 @@ import pytest
 
 import six
 
-from ipatests.util import create_test_api, assert_equal, raises, PluginTester
-from ipatests.data import unicode_str
-from ipalib import errors, Command
+from ipatests.util import assert_equal, raises, PluginTester
+from ipalib import errors
 from ipaserver import rpcserver
 
 if six.PY3:
@@ -36,7 +35,8 @@ if six.PY3:
 
 pytestmark = pytest.mark.tier0
 
-class StartResponse(object):
+
+class StartResponse:
     def __init__(self):
         self.reset()
 
@@ -63,7 +63,7 @@ def test_not_found():
     url = '/ipa/foo/stuff'
     assert_equal(
         f.not_found(None, s, url, None),
-        [t % dict(url='/ipa/foo/stuff')]
+        [(t % dict(url='/ipa/foo/stuff')).encode('utf-8')]
     )
     assert s.status == '404 Not Found'
     assert s.headers == [('Content-Type', 'text/html; charset=utf-8')]
@@ -73,7 +73,9 @@ def test_not_found():
     url ='&nbsp;' + '<script>do_bad_stuff();</script>'
     assert_equal(
         f.not_found(None, s, url, None),
-        [t % dict(url='&amp;nbsp;&lt;script&gt;do_bad_stuff();&lt;/script&gt;')]
+        [(t % dict(
+            url='&amp;nbsp;&lt;script&gt;do_bad_stuff();&lt;/script&gt;')
+        ).encode('utf-8')]
     )
     assert s.status == '404 Not Found'
     assert s.headers == [('Content-Type', 'text/html; charset=utf-8')]
@@ -87,7 +89,7 @@ def test_bad_request():
 
     assert_equal(
         f.bad_request(None, s, 'illegal request'),
-        [t % dict(message='illegal request')]
+        [(t % dict(message='illegal request')).encode('utf-8')]
     )
     assert s.status == '400 Bad Request'
     assert s.headers == [('Content-Type', 'text/html; charset=utf-8')]
@@ -101,7 +103,7 @@ def test_internal_error():
 
     assert_equal(
         f.internal_error(None, s, 'request failed'),
-        [t % dict(message='request failed')]
+        [(t % dict(message='request failed')).encode('utf-8')]
     )
     assert s.status == '500 Internal Server Error'
     assert s.headers == [('Content-Type', 'text/html; charset=utf-8')]
@@ -115,7 +117,7 @@ def test_unauthorized_error():
 
     assert_equal(
         f.unauthorized(None, s, 'unauthorized', 'password-expired'),
-        [t % dict(message='unauthorized')]
+        [(t % dict(message='unauthorized')).encode('utf-8')]
     )
     assert s.status == '401 Unauthorized'
     assert s.headers == [('Content-Type', 'text/html; charset=utf-8'),
@@ -134,7 +136,7 @@ def test_params_2_args_options():
     assert f([args, options]) == (args, options)
 
 
-class test_session(object):
+class test_session:
     klass = rpcserver.wsgi_dispatch
 
     def test_route(self):
@@ -195,7 +197,7 @@ class test_xmlserver(PluginTester):
     _plugin = rpcserver.xmlserver
 
     def test_marshaled_dispatch(self): # FIXME
-        (o, api, home) = self.instance('Backend', in_server=True)
+        self.instance('Backend', in_server=True)
 
 
 class test_jsonserver(PluginTester):
@@ -209,11 +211,10 @@ class test_jsonserver(PluginTester):
         """
         Test the `ipaserver.rpcserver.jsonserver.unmarshal` method.
         """
-        (o, api, home) = self.instance('Backend', in_server=True)
+        o, _api, _home = self.instance('Backend', in_server=True)
 
         # Test with invalid JSON-data:
         e = raises(errors.JSONError, o.unmarshal, 'this wont work')
-        assert isinstance(e.error, ValueError)
         if six.PY2:
             assert unicode(e.error) == 'No JSON object could be decoded'
         else:
@@ -257,7 +258,7 @@ class test_jsonserver(PluginTester):
         assert unicode(e.error) == 'params[1] (aka options) must be a dict'
 
         # Test with valid values:
-        args = (u'jdoe', )
+        args = [u'jdoe']
         options = dict(givenname=u'John', sn='Doe')
         d = dict(method=u'user_add', params=(args, options), id=18)
         assert o.unmarshal(json.dumps(d)) == (u'user_add', args, options, 18)
