@@ -20,3 +20,8 @@
 """
 Sub-package containing unit tests for `xmlrpc` package.
 """
+import ipatests.util
+
+
+ipatests.util.check_ipaclient_unittests()
+ipatests.util.check_no_ipaapi()  # also ignore in make fasttest
