@@ -18,17 +18,23 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-Test the `ipalib/plugins/idrange.py` module, and XML-RPC in general.
+Test the `ipaserver/plugins/idrange.py` module, and XML-RPC in general.
 """
 
 import six
 
-from ipalib import api, errors
-from ipatests.test_xmlrpc.xmlrpc_test import Declarative, fuzzy_uuid
+from ipalib import api, errors, messages
+from ipalib import constants
+from ipaplatform import services
+from ipatests.test_xmlrpc.xmlrpc_test import (
+    Declarative, fuzzy_uuid, Fuzzy, fuzzy_sequence_of)
 from ipatests.test_xmlrpc import objectclasses
 from ipatests.util import MockLDAP
 from ipapython.dn import DN
 from ipatests.test_xmlrpc.test_user_plugin import get_user_result
+from ipatests.test_xmlrpc.mock_trust import (
+    get_range_dn, get_trusted_dom_dict, get_trusted_dom_range_dict,
+    get_trust_dn)
 import pytest
 
 if six.PY3:
@@ -42,6 +48,12 @@ rid_shift = 0
 for idrange in api.Command['idrange_find']()['result']:
     size = int(idrange['ipaidrangesize'][0])
     base_id = int(idrange['ipabaseid'][0])
+    rtype = idrange['iparangetype'][0]
+
+    if rtype == 'ipa-local-subid' or base_id == constants.SUBID_RANGE_START:
+        # ignore subordinate id range. It would push values beyond uint32_t.
+        # There is plenty of space below SUBUID_RANGE_START.
+        continue
 
     id_end = base_id + size
     rid_end = 0
@@ -110,51 +122,11 @@ testrange8_size = 50
 testrange8_base_rid = rid_shift + 700
 testrange8_secondary_base_rid = rid_shift + 800
 
-# Helper functions definitions
-
-
-def get_range_dn(name):
-    format_str = "cn={name},cn=ranges,cn=etc,{basedn}"
-    data = dict(name=name, basedn=api.env.basedn)
-    return format_str.format(**data)
-
-
-def get_trust_dn(name):
-    format_str = "cn={name},cn=ad,cn=trusts,{basedn}"
-    data = dict(name=name, basedn=api.env.basedn)
-    return format_str.format(**data)
-
-
-def encode_mockldap_value(value):
-    value = str(value)
-    if six.PY3:
-        return value.encode('utf-8')
-    else:
-        return value
-
-
-def get_trusted_dom_range_dict(name, base_id, size, rangetype, base_rid, sid):
-    return dict(
-        objectClass=[b"ipaIDrange", b"ipatrustedaddomainrange"],
-        ipaBaseID=encode_mockldap_value("{base_id}".format(base_id=base_id)),
-        ipaBaseRID=encode_mockldap_value("{base_rid}".format(base_rid=base_rid)),
-        ipaIDRangeSize=encode_mockldap_value("{size}".format(size=size)),
-        ipaNTTrustedDomainSID=encode_mockldap_value("{sid}".format(sid=sid)),
-        ipaRangeType=encode_mockldap_value("{rangetype}".format(rangetype=rangetype)),
-        )
-
-
-def get_trusted_dom_dict(name, sid):
-    return dict(
-        objectClass=[b"ipaNTTrustedDomain", b"ipaIDobject", b"top"],
-        ipaNTFlatName=encode_mockldap_value(name.upper()),
-        ipaNTTrustedDomainSID=encode_mockldap_value(sid),
-        ipaNTSIDBlacklistIncoming=b'S-1-0',
-        ipaNTTrustPartner=encode_mockldap_value('{name}.mock'.format(name=name)),
-        ipaNTTrustType=b'2',
-        ipaNTTrustDirection=b'3',
-        ipaNTTrustAttributes=b'8',
-        )
+testrange9 = u'testrange9'
+testrange9_base_id = id_shift + 800
+testrange9_size = 50
+testrange9_base_rid = rid_shift + 800
+testrange9_secondary_base_rid = rid_shift + 1800
 
 # Domain ranges definitions
 
@@ -401,13 +373,45 @@ IPA_LOCAL_RANGE_MOD_ERR = (
     "domain. Run `ipa help idrange` for more information"
 )
 
+dirsrv_instance = services.knownservices.dirsrv.service_instance("")
+
+fuzzy_restart_messages = fuzzy_sequence_of(Fuzzy(type=dict))
+
 
 @pytest.mark.tier1
 class test_range(Declarative):
-    @classmethod
-    def setup_class(cls):
-        super(test_range, cls).setup_class()
-        cls.teardown_class()
+    mockldap = None
+
+    @pytest.fixture(autouse=True, scope="class")
+    def range_setup(self, request, declarative_setup):
+        cls = request.cls
+
+        def fin():
+            cls.mockldap = MockLDAP()
+
+            cls.mockldap.del_entry(domain2_dn)
+            cls.mockldap.del_entry(domain3_dn)
+            cls.mockldap.del_entry(domain4_dn)
+            cls.mockldap.del_entry(domain5_dn)
+            cls.mockldap.del_entry(domain6_dn)
+            cls.mockldap.del_entry(domain7_dn)
+
+            cls.mockldap.del_entry(domain1range1_dn)
+            cls.mockldap.del_entry(domain2range1_dn)
+            cls.mockldap.del_entry(domain2range2_dn)
+            cls.mockldap.del_entry(domain3range1_dn)
+            cls.mockldap.del_entry(domain3range2_dn)
+            cls.mockldap.del_entry(domain4range1_dn)
+            cls.mockldap.del_entry(domain5range1_dn)
+            cls.mockldap.del_entry(domain5range2_dn)
+            cls.mockldap.del_entry(domain6range1_dn)
+            cls.mockldap.del_entry(domain7range1_dn)
+            cls.mockldap.del_entry(trust_container_dn)
+            cls.mockldap.del_entry(trust_local_dn)
+            cls.mockldap.del_entry(smb_cont_dn)
+            cls.mockldap.unbind()
+
+        fin()
         cls.mockldap = MockLDAP()
         cls.mockldap.add_entry(trust_container_dn, trust_container_add)
         cls.mockldap.add_entry(smb_cont_dn, smb_cont_add)
@@ -431,35 +435,12 @@ class test_range(Declarative):
         cls.mockldap.add_entry(domain6range1_dn, domain6range1_add)
         cls.mockldap.unbind()
 
-    @classmethod
-    def teardown_class(cls):
-        cls.mockldap = MockLDAP()
-
-        cls.mockldap.del_entry(domain2_dn)
-        cls.mockldap.del_entry(domain3_dn)
-        cls.mockldap.del_entry(domain4_dn)
-        cls.mockldap.del_entry(domain5_dn)
-        cls.mockldap.del_entry(domain6_dn)
-        cls.mockldap.del_entry(domain7_dn)
-
-        cls.mockldap.del_entry(domain1range1_dn)
-        cls.mockldap.del_entry(domain2range1_dn)
-        cls.mockldap.del_entry(domain2range2_dn)
-        cls.mockldap.del_entry(domain3range1_dn)
-        cls.mockldap.del_entry(domain3range2_dn)
-        cls.mockldap.del_entry(domain4range1_dn)
-        cls.mockldap.del_entry(domain5range1_dn)
-        cls.mockldap.del_entry(domain5range2_dn)
-        cls.mockldap.del_entry(domain6range1_dn)
-        cls.mockldap.del_entry(domain7range1_dn)
-        cls.mockldap.del_entry(trust_container_dn)
-        cls.mockldap.del_entry(trust_local_dn)
-        cls.mockldap.del_entry(smb_cont_dn)
-        cls.mockldap.unbind()
+        request.addfinalizer(fin)
 
     cleanup_commands = [
         ('idrange_del', [testrange1, testrange2, testrange3, testrange4,
-                         testrange5, testrange6, testrange7, testrange8],
+                         testrange5, testrange6, testrange7, testrange8,
+                         testrange9],
                         {'continue': True}),
         ('user_del', [user1], {}),
         ('group_del', [group1], {}),
@@ -488,6 +469,11 @@ class test_range(Declarative):
                 ),
                 value=testrange1,
                 summary=u'Added ID range "%s"' % (testrange1),
+                messages=(
+                    messages.ServiceRestartRequired(
+                        service=dirsrv_instance,
+                        server='<all IPA servers>').to_dict(),
+                ),
             ),
         ),
 
@@ -526,6 +512,7 @@ class test_range(Declarative):
                     user1, u'Test', u'User1', 'add',
                     uidnumber=[unicode(user1_uid)],
                     gidnumber=[unicode(user1_uid)],
+                    objectclass=objectclasses.user_base + [u'mepOriginEntry'],
                 ),
             ),
         ),
@@ -612,12 +599,22 @@ class test_range(Declarative):
             ),
         ),
 
+        # Framework validation: mod local idrange with auto-private-groups
+        # is prohibited
+        dict(
+            desc=('Try to modify local range %r with --auto-private-groups'
+                  % (testrange1)),
+            command=('idrange_mod', [testrange1],
+                     dict(ipaautoprivategroups='true')),
+            expected=errors.ExecutionError(message=IPA_LOCAL_RANGE_MOD_ERR)
+        ),
 
         dict(
             desc='Delete ID range %r' % testrange1,
             command=('idrange_del', [testrange1], {}),
             expected=dict(
                 result=dict(failed=[]),
+                messages=fuzzy_restart_messages,
                 value=[testrange1],
                 summary=u'Deleted ID range "%s"' % testrange1,
             ),
@@ -647,6 +644,11 @@ class test_range(Declarative):
                 ),
                 value=testrange2,
                 summary=u'Added ID range "%s"' % (testrange2),
+                messages=(
+                    messages.ServiceRestartRequired(
+                        service=dirsrv_instance,
+                        server='<all IPA servers>').to_dict(),
+                ),
             ),
         ),
 
@@ -717,6 +719,7 @@ class test_range(Declarative):
             command=('idrange_del', [testrange2], {}),
             expected=dict(
                 result=dict(failed=[]),
+                messages=fuzzy_restart_messages,
                 value=[testrange2],
                 summary=u'Deleted ID range "%s"' % testrange2,
             ),
@@ -738,6 +741,22 @@ class test_range(Declarative):
                      'secondary-rid-base cannot be used together'),
         ),
 
+        # Testing framework validation: --auto-private-groups is prohibited
+        # with ipa-local range
+
+        dict(
+            desc='Local ID range %r with auto-private-groups' % (testrange8),
+            command=('idrange_add', [testrange8],
+                     dict(ipabaseid=testrange8_base_id,
+                          ipaidrangesize=testrange8_size,
+                          ipaautoprivategroups='true')),
+            expected=errors.ValidationError(
+                name='ID Range setup',
+                error='IPA Range type must be one of ipa-ad-trust '
+                      'or ipa-ad-trust-posix when '
+                      'auto-private-groups is specified'),
+        ),
+
         # Testing framework validation: --rid-base is prohibited with ipa-ad-posix
 
         dict(
@@ -754,6 +773,22 @@ class test_range(Declarative):
                       'type is ipa-ad-trust-posix'),
         ),
 
+        # Testing framework validation: --auto-private-groups can only be
+        # one of true, false, hybrid
+        dict(
+            desc=('Create ID range %r with bogus --auto-private-groups'
+                  % (domain7range1)),
+            command=('idrange_add', [domain7range1],
+                     dict(ipabaseid=domain7range1_base_id,
+                          ipaidrangesize=domain7range1_size,
+                          iparangetype=domain7range1_type,
+                          ipanttrusteddomainsid=domain7_sid,
+                          ipaautoprivategroups='bogus')),
+            expected=errors.ValidationError(
+                name='auto_private_groups',
+                error="must be one of 'true', 'false', 'hybrid'"),
+        ),
+
         dict(
             desc='Create ID range %r' % (domain7range1),
             command=('idrange_add', [domain7range1],
@@ -796,6 +831,7 @@ class test_range(Declarative):
                 result=dict(failed=[]),
                 value=[domain1range1],
                 summary=u'Deleted ID range "%s"' % domain1range1,
+                messages=fuzzy_restart_messages,
             ),
         ),
 
@@ -822,6 +858,7 @@ class test_range(Declarative):
             command=('idrange_mod', [domain3range2],
                      dict(ipabaseid=domain3range1_base_id)),
             expected=dict(
+                messages=fuzzy_restart_messages,
                 result=dict(
                     cn=[domain3range2],
                     ipabaseid=[unicode(domain3range1_base_id)],
@@ -887,6 +924,7 @@ class test_range(Declarative):
             command=('idrange_mod', [domain2range1],
                      dict(ipabaserid=domain5range1_base_rid)),
             expected=dict(
+                messages=fuzzy_restart_messages,
                 result=dict(
                     cn=[domain2range1],
                     ipabaseid=[unicode(domain2range1_base_id)],
@@ -915,4 +953,137 @@ class test_range(Declarative):
                      'range.'),
         ),
 
+        dict(
+            desc=('Modify ipa-ad-trust range %r with --auto-private-groups='
+                  'true' % (domain2range1)),
+            command=('idrange_mod', [domain2range1],
+                     dict(ipaautoprivategroups='true')),
+            expected=dict(
+                messages=fuzzy_restart_messages,
+                result=dict(
+                    cn=[domain2range1],
+                    ipabaseid=[unicode(domain2range1_base_id)],
+                    ipabaserid=[unicode(domain5range1_base_rid)],
+                    ipaidrangesize=[unicode(domain2range1_size)],
+                    ipanttrusteddomainsid=[unicode(domain2_sid)],
+                    iparangetyperaw=[u'ipa-ad-trust'],
+                    ipaautoprivategroups=[u'true'],
+                    iparangetype=[u'Active Directory domain range'],
+                ),
+                value=domain2range1,
+                summary=u'Modified ID range "%s"' % (domain2range1),
+            ),
+        ),
+
+        dict(
+            desc=('Modify ipa-ad-trust range %r with --auto-private-groups='
+                  'false' % (domain2range1)),
+            command=('idrange_mod', [domain2range1],
+                     dict(ipaautoprivategroups='false')),
+            expected=dict(
+                messages=fuzzy_restart_messages,
+                result=dict(
+                    cn=[domain2range1],
+                    ipabaseid=[unicode(domain2range1_base_id)],
+                    ipabaserid=[unicode(domain5range1_base_rid)],
+                    ipaidrangesize=[unicode(domain2range1_size)],
+                    ipanttrusteddomainsid=[unicode(domain2_sid)],
+                    iparangetyperaw=[u'ipa-ad-trust'],
+                    ipaautoprivategroups=[u'false'],
+                    iparangetype=[u'Active Directory domain range'],
+                ),
+                value=domain2range1,
+                summary=u'Modified ID range "%s"' % (domain2range1),
+            ),
+        ),
+
+        dict(
+            desc=('Modify ipa-ad-trust range %r with --auto-private-groups='
+                  'hybrid' % (domain2range1)),
+            command=('idrange_mod', [domain2range1],
+                     dict(ipaautoprivategroups='hybrid')),
+            expected=dict(
+                messages=fuzzy_restart_messages,
+                result=dict(
+                    cn=[domain2range1],
+                    ipabaseid=[unicode(domain2range1_base_id)],
+                    ipabaserid=[unicode(domain5range1_base_rid)],
+                    ipaidrangesize=[unicode(domain2range1_size)],
+                    ipanttrusteddomainsid=[unicode(domain2_sid)],
+                    iparangetyperaw=[u'ipa-ad-trust'],
+                    ipaautoprivategroups=[u'hybrid'],
+                    iparangetype=[u'Active Directory domain range'],
+                ),
+                value=domain2range1,
+                summary=u'Modified ID range "%s"' % (domain2range1),
+            ),
+        ),
+
+        dict(
+            desc=('Modify ipa-ad-trust range %r with --auto-private-groups='
+                  '<empty>' % (domain2range1)),
+            command=('idrange_mod', [domain2range1],
+                     dict(ipaautoprivategroups='')),
+            expected=dict(
+                messages=fuzzy_restart_messages,
+                result=dict(
+                    cn=[domain2range1],
+                    ipabaseid=[unicode(domain2range1_base_id)],
+                    ipabaserid=[unicode(domain5range1_base_rid)],
+                    ipaidrangesize=[unicode(domain2range1_size)],
+                    ipanttrusteddomainsid=[unicode(domain2_sid)],
+                    iparangetyperaw=[u'ipa-ad-trust'],
+                    iparangetype=[u'Active Directory domain range'],
+                ),
+                value=domain2range1,
+                summary=u'Modified ID range "%s"' % (domain2range1),
+            ),
+        ),
+
+        # Test for bug 6404
+        # if dom-name is empty, add should not fail
+
+        dict(
+            desc='Create ID range %r' % (testrange9),
+            command=('idrange_add', [testrange9],
+                     dict(ipanttrusteddomainname=None,
+                          ipabaseid=testrange9_base_id,
+                          ipaidrangesize=testrange9_size,
+                          ipabaserid=testrange9_base_rid,
+                          ipasecondarybaserid=testrange9_secondary_base_rid)),
+            expected=dict(
+                result=dict(
+                    dn=DN(('cn', testrange9), ('cn', 'ranges'), ('cn', 'etc'),
+                          api.env.basedn),
+                    cn=[testrange9],
+                    objectclass=[u'ipaIDrange', u'ipadomainidrange'],
+                    ipabaseid=[unicode(testrange9_base_id)],
+                    ipabaserid=[unicode(testrange9_base_rid)],
+                    ipasecondarybaserid=[
+                        unicode(testrange9_secondary_base_rid)],
+                    ipaidrangesize=[unicode(testrange9_size)],
+                    iparangetyperaw=[u'ipa-local'],
+                    iparangetype=[u'local domain range'],
+                ),
+                value=testrange9,
+                summary=u'Added ID range "%s"' % (testrange9),
+                messages=(
+                    messages.ServiceRestartRequired(
+                        service=dirsrv_instance,
+                        server='<all IPA servers>').to_dict(),
+                ),
+            ),
+        ),
+
+        dict(
+            desc='Delete ID range %r' % testrange9,
+            command=('idrange_del', [testrange9], {}),
+            expected=dict(
+                result=dict(failed=[]),
+                value=[testrange9],
+                summary=u'Deleted ID range "%s"' % testrange9,
+                messages=fuzzy_restart_messages,
+            ),
+        ),
+
     ]
